from abjad import enums
from abjad.system.FormatSpecification import FormatSpecification
from . import constants
from .PitchClass import PitchClass


class NumberedPitchClass(PitchClass):
    """
    Numbered pitch-class.

    ..  container:: example

        Initializes from number of semitones:

        >>> abjad.NumberedPitchClass(13)
        NumberedPitchClass(1)

    ..  container:: example

        Initializes from pitch name.

        >>> abjad.NumberedPitchClass('d')
        NumberedPitchClass(2)

    ..  container:: example

        Initializes from named pitch.

        >>> abjad.NumberedPitchClass(abjad.NamedPitch('g,'))
        NumberedPitchClass(7)

    ..  container:: example

        Initializes from numbered pitch.

        >>> abjad.NumberedPitchClass(abjad.NumberedPitch(15))
        NumberedPitchClass(3)

    ..  container:: example

        Initializes from named pitch-class.

        >>> abjad.NumberedPitchClass(abjad.NamedPitchClass('e'))
        NumberedPitchClass(4)

    ..  container:: example

        Initializes from pitch-class / octave string:

        >>> abjad.NumberedPitchClass('C#5')
        NumberedPitchClass(1)

    ..  container:: example

        Initializes from other numbered pitch-class:

        >>> abjad.NumberedPitchClass(abjad.NumberedPitchClass(9))
        NumberedPitchClass(9)

    ..  container:: example

        Initializes from note:

        >>> abjad.NumberedPitchClass(abjad.Note("a'8."))
        NumberedPitchClass(9)

    """

    ### CLASS VARIABLES ###

    __slots__ = (
        '_arrow',
        '_number',
        )

    ### INITIALIZER ###

    def __init__(self, number=0, *, arrow=None):
        super().__init__(number or 0)
        if arrow is not None:
            arrow = enums.VerticalAlignment.from_expr(arrow)
            if arrow is enums.Center:
                arrow = None
            self._arrow = arrow

    ### SPECIAL METHODS ###

    def __add__(self, argument):
        """
        Adds `argument` to numbered pitch-class.

        ..  container:: example

            >>> pitch_class = abjad.NumberedPitchClass(9)

            >>> pitch_class + abjad.NumberedInterval(0)
            NumberedPitchClass(9)

            >>> pitch_class + abjad.NumberedInterval(1)
            NumberedPitchClass(10)

            >>> pitch_class + abjad.NumberedInterval(2)
            NumberedPitchClass(11)

            >>> pitch_class + abjad.NumberedInterval(3)
            NumberedPitchClass(0)

        Returns new numbered pitch-class.
        """
        import abjad
        interval = abjad.NumberedInterval(argument)
        return type(self)(self.number + interval.number % 12)

    def __copy__(self, *arguments):
        """
        Copies numbered pitch-class.

        ..  container:: example

            >>> import copy
            >>> pitch_class = abjad.NumberedPitchClass(9)
            >>> copy.copy(pitch_class)
            NumberedPitchClass(9)

        Returns new numbered pitch-class.
        """
        return type(self)(self)

    def __eq__(self, argument):
        """
        Is true when `argument` is a numbered pitch-class with pitch-class
        number equal to that of this numbered pitch-class.

        ..  container:: example

            >>> pitch_class_1 = abjad.NumberedPitchClass(0)
            >>> pitch_class_2 = abjad.NumberedPitchClass(0)
            >>> pitch_class_3 = abjad.NumberedPitchClass(1)

            >>> pitch_class_1 == pitch_class_1
            True
            >>> pitch_class_1 == pitch_class_2
            True
            >>> pitch_class_1 == pitch_class_3
            False

            >>> pitch_class_2 == pitch_class_1
            True
            >>> pitch_class_2 == pitch_class_2
            True
            >>> pitch_class_2 == pitch_class_3
            False

            >>> pitch_class_3 == pitch_class_1
            False
            >>> pitch_class_3 == pitch_class_2
            False
            >>> pitch_class_3 == pitch_class_3
            True

        Returns true or false.
        """
        return super().__eq__(argument)

    def __format__(self, format_specification=''):
        """
        Formats numbered pitch-class.

        ..  container:: example

            >>> format(abjad.NumberedPitchClass(13))
            'abjad.NumberedPitchClass(1)'

        Set `format_specification` to `''`, `'lilypond'` or `'storage'`.

        Returns string.
        """
        return super().__format__(format_specification=format_specification)

    def __hash__(self):
        """
        Hashes numbered pitch-class.

        Required to be explicitly redefined on Python 3 if __eq__ changes.

        Returns integer.
        """
        return super().__hash__()

    def __lt__(self, argument):
        """
        Is true when `argument` is a numbered pitch-class with a pitch
        number greater than that of this numberd pitch-class.

        ..  container:: example

            Compares less than:

            >>> abjad.NumberedPitchClass(1) < abjad.NumberedPitchClass(2)
            True

        ..  container:: example

            Does not compare less than:

            >>> abjad.NumberedPitchClass(2) < abjad.NumberedPitchClass(1)
            False

        Raises type error when `argument` is not a numbered pitch-class.
        """
        if not isinstance(argument, type(self)):
            message = 'can not compare numbered pitch-class to {!r}.'
            message = message.format(argument)
            raise TypeError(message)
        return self.number < argument.number

    def __neg__(self):
        """
        Negates numbered pitch-class.

        ..  container:: example

            >>> pitch_class = abjad.NumberedPitchClass(9)
            >>> -pitch_class
            NumberedPitchClass(3)

        Returns new numbered pitch-class.
        """
        return type(self)(-self.number)

    def __radd__(self, argument):
        """
        Right-addition not defined on numbered pitch-classes.

        ..  container:: example

            >>> 1 + abjad.NumberedPitchClass(9)
            Traceback (most recent call last):
                ...
            NotImplementedError: right-addition not defined on NumberedPitchClass.

        Raises not implemented error.
        """
        message = 'right-addition not defined on {}.'
        message = message.format(type(self).__name__)
        raise NotImplementedError(message)

    def __str__(self):
        """
        Gets string representation of numbered pitch-class.

        Returns string.
        """
        return str(self.number)

    def __sub__(self, argument):
        """
        Subtracts `argument` from numbered pitch-class.

        Subtraction is defined against both numbered intervals
        and against other pitch-classes.

        ..  container:: example

            >>> abjad.NumberedPitchClass(6) - abjad.NumberedPitchClass(6)
            NumberedInversionEquivalentIntervalClass(0)

            >>> abjad.NumberedPitchClass(6) - abjad.NumberedPitchClass(7)
            NumberedInversionEquivalentIntervalClass(1)

            >>> abjad.NumberedPitchClass(7) - abjad.NumberedPitchClass(6)
            NumberedInversionEquivalentIntervalClass(1)

        ..  container:: example

            >>> abjad.NumberedPitchClass(6) - abjad.NumberedInterval(-1)
            NumberedPitchClass(5)

            >>> abjad.NumberedPitchClass(6) - abjad.NumberedInterval(0)
            NumberedPitchClass(6)

            >>> abjad.NumberedPitchClass(6) - abjad.NumberedInterval(1)
            NumberedPitchClass(5)

        Returns numbered inversion-equivalent interval-class.
        """
        import abjad
        if isinstance(argument, type(self)):
            interval_class_number = abs(
                self.number - argument.number
                )
            if 6 < interval_class_number:
                interval_class_number = 12 - interval_class_number
            return abjad.NumberedInversionEquivalentIntervalClass(
                interval_class_number)
        interval_class = abjad.NumberedInversionEquivalentIntervalClass(
            argument)
        return type(self)(self.number - interval_class.number % 12)

    ### PRIVATE METHODS ###

    def _apply_accidental(self, accidental=None):
        import abjad
        accidental = abjad.Accidental(accidental)
        semitones = self.number + accidental.semitones
        return type(self)(semitones)

    def _from_named_parts(self, dpc_number, alteration):
        number = constants._diatonic_pc_number_to_pitch_class_number[dpc_number]
        number += alteration
        self._from_number(number)

    def _from_number(self, number):
        self._arrow = None
        self._number = self._to_nearest_quarter_tone(number)

    def _from_pitch_or_pitch_class(self, pitch_or_pitch_class):
        self._arrow = pitch_or_pitch_class.arrow
        self._number = self._to_nearest_quarter_tone(float(pitch_or_pitch_class))

    def _get_alteration(self):
        dpc_number = self._get_diatonic_pc_number()
        pc_number = constants._diatonic_pc_number_to_pitch_class_number[dpc_number]
        return float(self) - pc_number

    def _get_diatonic_pc_name(self):
        return self.name[0]

    def _get_diatonic_pc_number(self):
        return constants._diatonic_pc_name_to_diatonic_pc_number[
            self._get_diatonic_pc_name()]

    def _get_format_specification(self):
        values = [self.number]
        return FormatSpecification(
            client=self,
            coerce_for_equality=True,
            storage_format_is_indented=False,
            storage_format_args_values=values,
            storage_format_kwargs_names=[],
            )

    def _get_lilypond_format(self):
        import abjad
        return format(abjad.NamedPitchClass(self), 'lilypond')

    ### PUBLIC PROPERTIES ###

    @property
    def accidental(self):
        """
        Gets accidental.

        ..  container:: example

            >>> abjad.NumberedPitchClass(1).accidental
            Accidental('sharp')

        Returns accidental.
        """
        import abjad
        return abjad.NamedPitch(self.number).accidental

    @property
    def arrow(self):
        """
        Gets arrow of numbered pitch-class.

        Returns up, down or none.
        """
        return self._arrow

    @property
    def name(self):
        """
        Gets name of numbered pitch-class.

        ..  container:: example

            >>> abjad.NumberedPitchClass(13).name
            'cs'

        Returns string.
        """
        return constants._pitch_class_number_to_pitch_class_name[self.number]

    @property
    def number(self):
        """
        Gets number.

        ..  container:: example

            >>> abjad.NumberedPitchClass(1).number
            1

        ..  container:: example

            >>> abjad.NumberedPitchClass(13).number
            1

        """
        return self._number

    @property
    def pitch_class_label(self):
        """
        Gets pitch-class / octave label.

        ..  container:: example

            >>> abjad.NumberedPitchClass(13).pitch_class_label
            'C#'

        Returns string.
        """
        return '{}{}'.format(
            self._get_diatonic_pc_name().upper(),
            self.accidental.symbol,
            )

    ### PUBLIC METHODS ###

    def invert(self, axis=None):
        """
        Inverts numbered pitch-class.

        ..  container:: example

            >>> for n in range(12):
            ...     pitch_class = abjad.NumberedPitchClass(n)
            ...     print(repr(pitch_class), repr(pitch_class.invert()))
            ...
            NumberedPitchClass(0) NumberedPitchClass(0)
            NumberedPitchClass(1) NumberedPitchClass(11)
            NumberedPitchClass(2) NumberedPitchClass(10)
            NumberedPitchClass(3) NumberedPitchClass(9)
            NumberedPitchClass(4) NumberedPitchClass(8)
            NumberedPitchClass(5) NumberedPitchClass(7)
            NumberedPitchClass(6) NumberedPitchClass(6)
            NumberedPitchClass(7) NumberedPitchClass(5)
            NumberedPitchClass(8) NumberedPitchClass(4)
            NumberedPitchClass(9) NumberedPitchClass(3)
            NumberedPitchClass(10) NumberedPitchClass(2)
            NumberedPitchClass(11) NumberedPitchClass(1)

        Interprets axis of inversion equal to pitch-class 0.

        Returns new numbered pitch-class.
        """
        import abjad
        axis = axis or abjad.NumberedPitch('c')
        axis = abjad.NumberedPitch(axis)
        this = abjad.NumberedPitch(self)
        interval = this - axis
        result = axis.transpose(interval)
        result = type(self)(result)
        return result

    def multiply(self, n=1):
        """
        Multiplies pitch-class number by `n`.

        ..  container:: example

            >>> for n in range(12):
            ...     pitch_class = abjad.NumberedPitchClass(n)
            ...     print(repr(pitch_class), repr(pitch_class.multiply(5)))
            ...
            NumberedPitchClass(0) NumberedPitchClass(0)
            NumberedPitchClass(1) NumberedPitchClass(5)
            NumberedPitchClass(2) NumberedPitchClass(10)
            NumberedPitchClass(3) NumberedPitchClass(3)
            NumberedPitchClass(4) NumberedPitchClass(8)
            NumberedPitchClass(5) NumberedPitchClass(1)
            NumberedPitchClass(6) NumberedPitchClass(6)
            NumberedPitchClass(7) NumberedPitchClass(11)
            NumberedPitchClass(8) NumberedPitchClass(4)
            NumberedPitchClass(9) NumberedPitchClass(9)
            NumberedPitchClass(10) NumberedPitchClass(2)
            NumberedPitchClass(11) NumberedPitchClass(7)

        Returns new numbered pitch-class.
        """
        return type(self)(n * self.number)

    def transpose(self, n=0):
        """
        Transposes numbered pitch-class by index `n`.

        ..  container:: example

            >>> for n in range(12):
            ...     pitch_class = abjad.NumberedPitchClass(n)
            ...     print(repr(pitch_class), repr(pitch_class.transpose(-13)))
            ...
            NumberedPitchClass(0) NumberedPitchClass(11)
            NumberedPitchClass(1) NumberedPitchClass(0)
            NumberedPitchClass(2) NumberedPitchClass(1)
            NumberedPitchClass(3) NumberedPitchClass(2)
            NumberedPitchClass(4) NumberedPitchClass(3)
            NumberedPitchClass(5) NumberedPitchClass(4)
            NumberedPitchClass(6) NumberedPitchClass(5)
            NumberedPitchClass(7) NumberedPitchClass(6)
            NumberedPitchClass(8) NumberedPitchClass(7)
            NumberedPitchClass(9) NumberedPitchClass(8)
            NumberedPitchClass(10) NumberedPitchClass(9)
            NumberedPitchClass(11) NumberedPitchClass(10)

        Returns new numbered pitch-class.
        """
        return type(self)(self.number + n)



import functools
import math
import numbers
import re
from abjad.system.AbjadValueObject import AbjadValueObject
from abjad.system.FormatSpecification import FormatSpecification
from . import constants


@functools.total_ordering
class Octave(AbjadValueObject):
    """
    Octave.

    ..  container:: example:

        Initializes octave from integer:

        >>> abjad.Octave(4)
        Octave(4)

    ..  container:: example

        Initializes octave from octave-tick string:

        >>> abjad.Octave(",,")
        Octave(1)

    ..  container:: example

        Initializes octave from named pitch:

        >>> abjad.Octave(abjad.NamedPitch("cs''"))
        Octave(5)

    ..  container:: example

        Initializes octave from other octave:

        >>> abjad.Octave(abjad.Octave(2))
        Octave(2)

    """

    ### CLASS VARIABLES ###

    __slots__ = (
        '_number',
        )

    ### INITIALIZER ###

    def __init__(self, number=4):
        import abjad
        if number is None:
            number = 4
        elif isinstance(number, str):
            match = constants._comprehensive_octave_regex.match(number)
            if match is None:
                try:
                    pitch = abjad.NamedPitch(number)
                    number = pitch.octave.number
                except Exception:
                    message = 'can not instantiate {}: {!r}.'
                    message = message.format(type(self), number)
                    raise ValueError(message)
            else:
                group_dict = match.groupdict()
                number = 3
                if group_dict['octave_number']:
                    number = int(group_dict['octave_number'])
                elif group_dict['octave_tick']:
                    if group_dict['octave_tick'].startswith("'"):
                        number += group_dict['octave_tick'].count("'")
                    else:
                        number -= group_dict['octave_tick'].count(',')
        elif isinstance(number, numbers.Number):
            number = int(number)
        elif hasattr(number, 'octave'):
            number = number.octave.number
        elif isinstance(number, type(self)):
            number = number.number
        else:
            try:
                pitch = abjad.NamedPitch(number)
                number = pitch.octave.number
            except Exception:
                message = 'can not instantiate {}: {!r}.'
                message = message.format(type(self), number)
                raise ValueError(message)
        self._number = number

    ### SPECIAL METHODS ###

    def __eq__(self, argument):
        """
        Is true when `argument` is octave with same octave number.

        ..  container:: example

            >>> octave_1 = abjad.Octave(4)
            >>> octave_2 = abjad.Octave(4)
            >>> octave_3 = abjad.Octave(5)

            >>> octave_1 == octave_1
            True
            >>> octave_1 == octave_2
            True
            >>> octave_1 == octave_3
            False

            >>> octave_2 == octave_1
            True
            >>> octave_2 == octave_2
            True
            >>> octave_2 == octave_3
            False

            >>> octave_3 == octave_1
            False
            >>> octave_3 == octave_2
            False
            >>> octave_3 == octave_3
            True

        Returns true or false.
        """
        return super().__eq__(argument)

    def __float__(self):
        """
        Get octave number as float.

        Returns float.
        """
        return float(self.number)

    def __hash__(self):
        """
        Hashes octave.

        Returns integer.
        """
        return super().__hash__()

    def __int__(self):
        """
        Get octave number integer.

        Returns integer.
        """
        return int(self.number)

    def __lt__(self, argument):
        """
        Is true when octave is less than `argument`.

        ..  container:: example

            >>> octave_1 = abjad.Octave(4)
            >>> octave_2 = abjad.Octave(4)
            >>> octave_3 = abjad.Octave(5)

            >>> octave_1 < octave_1
            False
            >>> octave_1 < octave_2
            False
            >>> octave_1 < octave_3
            True

            >>> octave_2 < octave_1
            False
            >>> octave_2 < octave_2
            False
            >>> octave_2 < octave_3
            True

            >>> octave_3 < octave_1
            False
            >>> octave_3 < octave_2
            False
            >>> octave_3 < octave_3
            False

        Returns true or false.
        """
        try:
            argument = type(self)(argument)
        except:
            False
        return self.number < argument.number

    def __str__(self):
        """
        Gets string representation of octave.

        Defined equal to LilyPond octave / tick representation of octave.

        ..  container:: example

            >>> str(abjad.Octave(4))
            "'"

            >>> str(abjad.Octave(1))
            ',,'

            >>> str(abjad.Octave(3))
            ''

        Returns string.
        """
        return self.ticks

    ### PRIVATE METHODS ###

    def _get_format_specification(self):
        return FormatSpecification(
            client=self,
            repr_is_indented=False,
            storage_format_is_indented=False,
            storage_format_args_values=[self.number],
            storage_format_kwargs_names=[],
            )

    @classmethod
    def _is_tick_string(class_, argument):
        if not isinstance(argument, str):
            return False
        return bool(class_._octave_tick_regex.match(argument))

    ### PUBLIC PROPERTIES ###

    @property
    def number(self):
        """
        Gets octave number.

        ..  container:: example

            >>> abjad.Octave(5).number
            5

        Returns integer.
        """
        return self._number

    @property
    def pitch_number(self):
        """
        Gets pitch number of first note in octave.

        ..  container:: example

            >>> abjad.Octave(4).pitch_number
            0

            >>> abjad.Octave(5).pitch_number
            12

            >>> abjad.Octave(3).pitch_number
            -12

        Returns integer.
        """
        return (self.number - 4) * 12

    @property
    def pitch_range(self):
        """
        Gets pitch range of octave.

        ..  container:: example

            >>> abjad.Octave(5).pitch_range
            PitchRange('[C5, C6)')

        Returns pitch range.
        """
        import abjad
        return abjad.PitchRange(
            '[C{}, C{})'.format(
                self.number,
                self.number + 1,
                ))

    @property
    def ticks(self):
        """
        Gets LilyPond octave tick string.

        ..  container:: example

            >>> for i in range(-1, 9):
            ...     print(i, abjad.Octave(i).ticks)
            -1 ,,,,
            0  ,,,
            1  ,,
            2  ,
            3
            4  '
            5  ''
            6  '''
            7  ''''
            8  '''''

        Returns string.
        """
        if 3 < self.number:
            return "'" * (self.number - 3)
        elif self.number < 3:
            return ',' * abs(3 - self.number)
        return ''

    ### PUBLIC METHODS ###

    @classmethod
    def from_pitch(class_, pitch):
        """Makes octave from `pitch`.

        ..  container:: example

            >>> abjad.Octave.from_pitch('cs')
            Octave(3)

            >>> abjad.Octave.from_pitch("cs'")
            Octave(4)

            >>> abjad.Octave.from_pitch(1)
            Octave(4)

            >>> abjad.Octave.from_pitch(13)
            Octave(5)

        Returns integer.
        """
        import abjad
        if isinstance(pitch, numbers.Number):
            number = int(math.floor(pitch / 12)) + 4
            return class_(number)
        if isinstance(pitch, abjad.NamedPitch):
            name = pitch.name
        elif isinstance(pitch, str):
            name = pitch
        else:
            raise TypeError(pitch)
        match = re.match('^([a-z]+)(\,*|\'*)$', name)
        assert match is not None, repr(match)
        name, ticks = match.groups()
        return class_(ticks)



import copy
from abjad import mathtools
from .Leaf import Leaf


class Chord(Leaf):
    """
    Chord.

    ..  container:: example

        >>> chord = abjad.Chord("<e' cs'' f''>4")
        >>> abjad.show(chord) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(chord)
            <e' cs'' f''>4

    """

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Leaves'

    __slots__ = (
        '_note_heads',
        )

    ### INITIALIZER ###

    def __init__(self, *arguments):
        import abjad
        from abjad.ly import drums
        assert len(arguments) in (0, 1, 2)
        self._note_heads = abjad.NoteHeadList(client=self)
        if len(arguments) == 1 and isinstance(arguments[0], str):
            string = '{{ {} }}'.format(arguments[0])
            parsed = abjad.parse(string)
            assert len(parsed) == 1 and isinstance(parsed[0], Leaf)
            arguments = [parsed[0]]
        are_cautionary = []
        are_forced = []
        are_parenthesized = []
        if len(arguments) == 1 and isinstance(arguments[0], Leaf):
            leaf = arguments[0]
            written_pitches = []
            written_duration = leaf.written_duration
            if 'written_pitch' in dir(leaf):
                written_pitches.append(leaf.note_head.written_pitch)
                are_cautionary = [leaf.note_head.is_cautionary]
                are_forced = [leaf.note_head.is_forced]
                are_parenthesized = [leaf.note_head.is_parenthesized]
            elif 'written_pitches' in dir(leaf):
                written_pitches.extend(
                    x.written_pitch for x in leaf.note_heads
                    )
                are_cautionary = [x.is_cautionary for x in leaf.note_heads]
                are_forced = [x.is_forced for x in leaf.note_heads]
                are_parenthesized = [
                    x.is_parenthesized for x in
                    leaf.note_heads
                    ]
        elif len(arguments) == 2:
            written_pitches, written_duration = arguments
            if isinstance(written_pitches, str):
                written_pitches = [x for x in written_pitches.split() if x]
            elif isinstance(written_pitches, type(self)):
                written_pitches = written_pitches.written_pitches
        elif len(arguments) == 0:
            written_pitches = [0, 4, 7]
            written_duration = abjad.Duration(1, 4)
        else:
            message = 'can not initialize chord from {!r}.'
            message = message.format(arguments)
            raise ValueError(message)
        Leaf.__init__(self, written_duration)
        if not are_cautionary:
            are_cautionary = [None] * len(written_pitches)
        if not are_forced:
            are_forced = [None] * len(written_pitches)
        if not are_parenthesized:
            are_parenthesized = [None] * len(written_pitches)
        for written_pitch, is_cautionary, is_forced, is_parenthesized in zip(
            written_pitches, are_cautionary, are_forced, are_parenthesized):
            if not is_cautionary:
                is_cautionary = None
            if not is_forced:
                is_forced = None
            if not is_parenthesized:
                is_parenthesized = None
            if written_pitch not in drums:
                note_head = abjad.NoteHead(
                    written_pitch=written_pitch,
                    is_cautionary=is_cautionary,
                    is_forced=is_forced,
                    is_parenthesized=is_parenthesized,
                    )
            else:
                note_head = abjad.DrumNoteHead(
                    written_pitch=written_pitch,
                    is_cautionary=is_cautionary,
                    is_forced=is_forced,
                    is_parenthesized=is_parenthesized,
                    )
            self._note_heads.append(note_head)
        if len(arguments) == 1 and isinstance(arguments[0], Leaf):
            self._copy_override_and_set_from_leaf(arguments[0])

    ### SPECIAL METHODS ###

    def __copy__(self, *arguments):
        """
        Shallow copies chord.

        Returns new chord.
        """
        new_chord = Leaf.__copy__(self, *arguments)
        new_chord.note_heads[:] = []
        for note_head in self.note_heads:
            note_head = copy.copy(note_head)
            new_chord.note_heads.append(note_head)
        return new_chord

    def __getnewargs__(self):
        """
        Gets new chord arguments.

        Returns pair.
        """
        return self.written_pitches, self.written_duration

    ### PRIVATE METHODS ###

    def _format_before_slot(self, bundle):
        import abjad
        result = []
        result.append(self._format_grace_body())
        result.append(('comments', bundle.before.comments))
        commands = bundle.before.commands
        if abjad.inspect(self).has_indicator(abjad.Tremolo):
            tremolo_command = self._format_repeat_tremolo_command()
            commands = list(commands)
            commands.append(tremolo_command)
            commands = tuple(commands)
        result.append(('commands', commands))
        result.append(('indicators', bundle.before.indicators))
        result.append(('grob overrides', bundle.grob_overrides))
        result.append(('context settings', bundle.context_settings))
        result.append(('spanners', bundle.before.spanners))
        return result

    def _format_close_brackets_slot(self, bundle):
        import abjad
        result = []
        if abjad.inspect(self).has_indicator(abjad.Tremolo):
            brackets_close = ['}']
            result.append([('close brackets', ''), brackets_close])
        return result

    def _format_leaf_nucleus(self):
        import abjad
        indent = abjad.LilyPondFormatManager.indent
        result = []
        note_heads = self.note_heads
        if any('\n' in format(x) for x in note_heads):
            for note_head in note_heads:
                current_format = format(note_head)
                format_list = current_format.split('\n')
                format_list = [indent + x for x in format_list]
                result.extend(format_list)
            result.insert(0, '<')
            result.append('>')
            result = '\n'.join(result)
            result += str(self._get_formatted_duration())
        elif abjad.inspect(self).has_indicator(abjad.Tremolo):
            reattack_duration = self._get_tremolo_reattack_duration()
            duration_string = reattack_duration.lilypond_duration_string
            durated_pitches = []
            for note_head in note_heads:
                durated_pitch = format(note_head) + duration_string
                durated_pitches.append(durated_pitch)
            tremolo = abjad.inspect(self).indicator(abjad.Tremolo)
            if tremolo.is_slurred:
                durated_pitches[0] = durated_pitches[0] + r' \('
                durated_pitches[-1] = durated_pitches[-1] + r' \)'
            result = ' '.join(durated_pitches)
        else:
            result.extend([format(_) for _ in note_heads])
            result = '<%s>%s' % (
                ' '.join(result),
                self._get_formatted_duration(),
                )
        # single string, but wrapped in list bc contribution
        return ['nucleus', [result]]

    def _format_open_brackets_slot(self, bundle):
        import abjad
        result = []
        if abjad.inspect(self).has_indicator(abjad.Tremolo):
            brackets_open = ['{']
            result.append([('open brackets', ''), brackets_open])
        return result

    def _format_repeat_tremolo_command(self):
        import abjad
        tremolo = abjad.inspect(self).indicator(abjad.Tremolo)
        reattack_duration = self._get_tremolo_reattack_duration()
        repeat_count = self.written_duration / reattack_duration / 2
        if not mathtools.is_integer_equivalent(repeat_count):
            message = 'can not tremolo duration {} with {} beams.'
            message = message.format(self.written_duration, tremolo.beam_count)
            raise Exception(message)
        repeat_count = int(repeat_count)
        command = r'\repeat tremolo {}'.format(repeat_count)
        return command

    def _get_compact_representation(self):
        return '<{}>{}'.format(
            self._get_summary(),
            self._get_formatted_duration(),
            )

    def _get_compact_representation_with_tie(self):
        logical_tie = self._get_logical_tie()
        if 1 < len(logical_tie) and self is not logical_tie[-1]:
            #return '{} ~'.format(self._get_body()[0])
            return '{} ~'.format(self._get_compact_representation())
        else:
            #return self._get_body()[0]
            return self._get_compact_representation()

    def _get_sounding_pitches(self):
        import abjad
        if 'sounding pitch' in abjad.inspect(self).indicators(str):
            return self.written_pitches
        else:
            instrument = self._get_effective(abjad.Instrument)
            if instrument:
                sounding_pitch = instrument.middle_c_sounding_pitch
            else:
                sounding_pitch = abjad.NamedPitch('C4')
            interval = abjad.NamedPitch('C4') - sounding_pitch
            sounding_pitches = [
                interval.transpose(pitch)
                for pitch in self.written_pitches
                ]
            return tuple(sounding_pitches)

    def _get_summary(self):
        return ' '.join([str(x) for x in self.note_heads])

    def _get_tremolo_reattack_duration(self):
        import abjad
        tremolos = abjad.inspect(self).indicators(abjad.Tremolo)
        if not tremolos:
            return
        tremolo = tremolos[0]
        exponent = 2 + tremolo.beam_count
        denominator = 2 ** exponent
        reattack_duration = abjad.Duration(1, denominator)
        return reattack_duration

    ### PUBLIC PROPERTIES ###

    @property
    def note_heads(self):
        r"""
        Gets note-heads in chord.

        ..  container:: example

            Gets note-heads in chord:

            >>> chord = abjad.Chord("<g' c'' e''>4")
            >>> abjad.show(chord) # doctest: +SKIP

            >>> abjad.f(chord.note_heads)
            abjad.NoteHeadList(
                [
                    abjad.NoteHead(
                        written_pitch=abjad.NamedPitch("g'"),
                        ),
                    abjad.NoteHead(
                        written_pitch=abjad.NamedPitch("c''"),
                        ),
                    abjad.NoteHead(
                        written_pitch=abjad.NamedPitch("e''"),
                        ),
                    ]
                )

        ..  container:: example

            Sets note-heads with pitch names:

            >>> chord = abjad.Chord("<g' c'' e''>4")
            >>> abjad.show(chord) # doctest: +SKIP

            >>> chord.note_heads = "c' d' fs'"
            >>> abjad.show(chord) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(chord)
                <c' d' fs'>4

        ..  container:: example

            Sets note-heads with pitch numbers:

                >>> chord = abjad.Chord("<g' c'' e''>4")
                >>> abjad.show(chord) # doctest: +SKIP

            >>> chord.note_heads = [16, 17, 19]
            >>> abjad.show(chord) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(chord)
                <e'' f'' g''>4

        Set note-heads with any iterable.

        Returns note-head list.
        """
        return self._note_heads

    @note_heads.setter
    def note_heads(self, note_heads):
        self._note_heads[:] = []
        if isinstance(note_heads, str):
            note_heads = note_heads.split()
        self.note_heads.extend(note_heads)

    @property
    def written_duration(self):
        """
        Gets written duration of chord.

        ..  container:: example

            Get written duration:

            >>> chord = abjad.Chord("<e' cs'' f''>4")
            >>> abjad.show(chord) # doctest: +SKIP

            >>> chord.written_duration
            Duration(1, 4)

        ..  container:: example

            Set written duration:

            >>> chord = abjad.Chord("<e' cs'' f''>4")
            >>> abjad.show(chord) # doctest: +SKIP

            >>> chord.written_duration = abjad.Duration(1, 16)
            >>> abjad.show(chord) # doctest: +SKIP

        Set duration.

        Returns duration.
        """
        return Leaf.written_duration.fget(self)

    @written_duration.setter
    def written_duration(self, argument):
        Leaf.written_duration.fset(self, argument)

    @property
    def written_pitches(self):
        """
        Written pitches in chord.

        ..  container:: example

            Get written pitches:

            >>> chord = abjad.Chord("<g' c'' e''>4")
            >>> abjad.show(chord) # doctest: +SKIP

            >>> chord.written_pitches
            PitchSegment("g' c'' e''")

        ..  container:: example

            Set written pitches with pitch names:

            >>> chord = abjad.Chord("<e' g' c''>4")
            >>> abjad.show(chord) # doctest: +SKIP

            >>> chord.written_pitches = "f' b' d''"
            >>> abjad.show(chord) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(chord)
                <f' b' d''>4

            >>> chord.written_pitches
            PitchSegment("f' b' d''")

        Set written pitches with any iterable.

        Returns tuple.
        """
        import abjad
        return abjad.PitchSegment(
            items=(note_head.written_pitch for note_head in self.note_heads),
            item_class=abjad.NamedPitch,
            )

    @written_pitches.setter
    def written_pitches(self, pitches):
        self.note_heads = pitches



import typing
from abjad import enums
from abjad.lilypondnames.LilyPondTweakManager import LilyPondTweakManager
from abjad.system.AbjadValueObject import AbjadValueObject
from abjad.system.LilyPondFormatBundle import LilyPondFormatBundle
from abjad.utilities.String import String


class Staccato(AbjadValueObject):
    r"""
    Staccato.

    ..  container:: example

        Attached to a single note:

        >>> note = abjad.Note("c'4")
        >>> staccato = abjad.Staccato()
        >>> abjad.attach(staccato, note)
        >>> abjad.show(note) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(note)
            c'4
            \staccato

    ..  container:: example

        Attached to notes in a staff:

        >>> staff = abjad.Staff("c'8 d' e' f' g' a' b' c''")
        >>> abjad.attach(abjad.Beam(), staff[:4])
        >>> abjad.attach(abjad.Beam(), staff[4:])
        >>> abjad.attach(abjad.Staccato(), staff[3])
        >>> abjad.attach(abjad.Staccato(), staff[7])
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            {
                c'8
                [
                d'8
                e'8
                f'8
                ]
                \staccato
                g'8
                [
                a'8
                b'8
                c''8
                ]
                \staccato
            }

    """

    ### CLASS VARIABLES ###

    __slots__ = (
        '_direction',
        '_tweaks',
        )

    _format_slot: enums.HorizontalAlignment = enums.Right

    _time_orientation: enums.HorizontalAlignment = enums.Right

    ### INITIALIZER ###

    def __init__(
        self,
        *,
        direction: enums.VerticalAlignment = None,
        tweaks: typing.Union[
            typing.List[typing.Tuple], LilyPondTweakManager] = None,
        ) -> None:
        direction_ = String.to_tridirectional_ordinal_constant(direction)
        if direction_ is not None:
            assert isinstance(direction_, enums.VerticalAlignment), repr(direction_)
            directions = (enums.Up, enums.Down, enums.Center, None)
            assert direction_ in directions, repr(direction_)
        self._direction = direction_
        self._tweaks = None
        LilyPondTweakManager.set_tweaks(self, tweaks)

    ### SPECIAL METHODS ###

    def __str__(self) -> str:
        r"""
        Gets string representation of staccato.

        ..  container:: example

            >>> str(abjad.Staccato())
            '\\staccato'

        """
        return r'\staccato'

    ### PRIVATE METHODS ###

    def _get_lilypond_format(self):
        return str(self)

    def _get_lilypond_format_bundle(self, component=None):
        bundle = LilyPondFormatBundle()
        if self.tweaks:
            tweaks = self.tweaks._list_format_contributions()
            bundle.after.commands.extend(tweaks)
        bundle.after.commands.append(self._get_lilypond_format())
        return bundle

    ### PUBLIC PROPERTIES ###

    @property
    def direction(self) -> typing.Optional[enums.VerticalAlignment]:
        """
        Gets direction of articulation.

        ..  container:: example

            Without direction:

            >>> abjad.Staccato().direction is None
            True

            >>> abjad.Staccato(direction=abjad.Up).direction
            Up

        """
        return self._direction

    @property
    def tweaks(self) -> typing.Optional[LilyPondTweakManager]:
        r"""
        Gets tweaks

        ..  container:: example

            >>> note = abjad.Note("c'4")
            >>> staccato = abjad.Staccato()
            >>> abjad.tweak(staccato).color = 'blue'
            >>> abjad.attach(staccato, note)
            >>> abjad.show(note) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(note)
                c'4
                - \tweak color #blue
                \staccato

        ..  container:: example

            >>> note = abjad.Note("c'4")
            >>> staccato = abjad.Staccato(tweaks=[('color', 'blue')])
            >>> abjad.attach(staccato, note)
            >>> abjad.show(note) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(note)
                c'4
                - \tweak color #blue
                \staccato

        """
        return self._tweaks



import abc
from abjad.system.AbjadObject import AbjadObject


class ContextManager(AbjadObject):
    """
    An abstract context manager class.
    """

    ### CLASS VARIABLES ###

    __slots__ = ()

    ### INTIALIZER ###

    @abc.abstractmethod
    def __init__(self):
        raise NotImplementedError

    ### SPECIAL METHODS ###

    @abc.abstractmethod
    def __enter__(self):
        """
        Enters context manager.
        """
        raise NotImplementedError

    @abc.abstractmethod
    def __exit__(self, exc_type, exc_value, traceback):
        """
        Exits context manager.
        """
        raise NotImplementedError



from .Duration import Duration


class Multiplier(Duration):
    """
    Multiplier.

    ..  container:: example

        Initializes from integer numerator:

        >>> abjad.Multiplier(3)
        Multiplier(3, 1)

    ..  container:: example

        Initializes from integer numerator and denominator:

        >>> abjad.Multiplier(3, 16)
        Multiplier(3, 16)

    ..  container:: example

        Initializes from integer-equivalent numeric numerator:

        >>> abjad.Multiplier(3.0)
        Multiplier(3, 1)

    ..  container:: example

        Initializes from integer-equivalent numeric numerator and denominator:

        >>> abjad.Multiplier(3.0, 16)
        Multiplier(3, 16)

    ..  container:: example

        Initializes from integer-equivalent singleton:

        >>> abjad.Multiplier((3,))
        Multiplier(3, 1)

    ..  container:: example

        Initializes from integer-equivalent pair:

        >>> abjad.Multiplier((3, 16))
        Multiplier(3, 16)

    ..  container:: example

        Initializes from other duration:

        >>> abjad.Multiplier(abjad.Duration(3, 16))
        Multiplier(3, 16)

    ..  container:: example

        Intializes from fraction:

        >>> abjad.Multiplier(abjad.Fraction(3, 16))
        Multiplier(3, 16)

    ..  container:: example

        Initializes from solidus string:

        >>> abjad.Multiplier('3/16')
        Multiplier(3, 16)

    ..  container:: example

        Initializes from nonreduced fraction:

        >>> abjad.Multiplier(abjad.NonreducedFraction(6, 32))
        Multiplier(3, 16)

    ..  container:: example

        Multipliers inherit from built-in fraction:

        >>> isinstance(abjad.Multiplier(3, 16), abjad.Fraction)
        True

    ..  container:: example

        Multipliers are numbers:

        >>> import numbers

        >>> isinstance(abjad.Multiplier(3, 16), numbers.Number)
        True

    ..  container:: example

        Attaching a multiplier to a score component multiplies that component's
        duration:

        >>> note = abjad.Note("c'1")
        >>> abjad.f(note)
        c'1

        >>> multiplier = abjad.Multiplier(3, 8)
        >>> abjad.attach(multiplier, note)
        >>> abjad.f(note)
        c'1 * 3/8

    """

    ### CLASS VARIABLES ###

    __slots__ = ()

    ### SPECIAL METHODS ###

    def __mul__(self, *arguments):
        """
        Multiplier times duration gives duration.

        Returns duration.
        """
        if len(arguments) == 1 and type(arguments[0]) is Duration:
            return Duration(Duration.__mul__(self, *arguments))
        else:
            return Duration.__mul__(self, *arguments)

    ### PUBLIC PROPERTIES ###

    @staticmethod
    def from_dot_count(dot_count: int) -> 'Multiplier':
        """
        Makes multiplier from ``dot_count``.

        ..  container:: example

            >>> abjad.Multiplier.from_dot_count(0)
            Multiplier(1, 1)

            >>> abjad.Multiplier.from_dot_count(1)
            Multiplier(3, 2)

            >>> abjad.Multiplier.from_dot_count(2)
            Multiplier(7, 4)

            >>> abjad.Multiplier.from_dot_count(3)
            Multiplier(15, 8)

            >>> abjad.Multiplier.from_dot_count(4)
            Multiplier(31, 16)

        """
        assert isinstance(dot_count, int), repr(dot_count)
        assert 0 <= dot_count, repr(dot_count)
        denominator = 2 ** dot_count
        numerator = 2 ** (dot_count + 1) - 1
        return Multiplier(numerator, denominator)

    def normalized(self):
        """
        Is true when mutliplier is greater than ``1/2`` and less than ``2``.

        ..  container:: example

            >>> abjad.Multiplier(3, 2).normalized()
            True

        Returns true or false.
        """
        return type(self)(1, 2) < self < type(self)(2)



import collections
from abjad.utilities.OrderedDict import OrderedDict
from .ScoreTemplate import ScoreTemplate


class GroupedRhythmicStavesScoreTemplate(ScoreTemplate):
    r"""
    Grouped rhythmic staves score template.

    ..  container:: example

        One voice per staff:

        >>> class_ = abjad.GroupedRhythmicStavesScoreTemplate
        >>> template_1 = class_(staff_count=4)
        >>> abjad.show(template_1) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(template_1.__illustrate__()[abjad.Score])
            \context Score = "Grouped Rhythmic Staves Score"
            <<
                \context StaffGroup = "Grouped Rhythmic Staves Staff Group"
                <<
                    \context RhythmicStaff = "Staff 1"
                    {
                        \context Voice = "Voice 1"
                        {
                            \clef "percussion" %! ST3
                            s1
                        }
                    }
                    \context RhythmicStaff = "Staff 2"
                    {
                        \context Voice = "Voice 2"
                        {
                            \clef "percussion" %! ST3
                            s1
                        }
                    }
                    \context RhythmicStaff = "Staff 3"
                    {
                        \context Voice = "Voice 3"
                        {
                            \clef "percussion" %! ST3
                            s1
                        }
                    }
                    \context RhythmicStaff = "Staff 4"
                    {
                        \context Voice = "Voice 4"
                        {
                            \clef "percussion" %! ST3
                            s1
                        }
                    }
                >>
            >>

        >>> score = template_1()
        >>> abjad.show(score) # doctest: +SKIP

        >>> abjad.f(score)
        \context Score = "Grouped Rhythmic Staves Score"
        <<
            \context StaffGroup = "Grouped Rhythmic Staves Staff Group"
            <<
                \context RhythmicStaff = "Staff 1"
                {
                    \context Voice = "Voice 1"
                    {
                    }
                }
                \context RhythmicStaff = "Staff 2"
                {
                    \context Voice = "Voice 2"
                    {
                    }
                }
                \context RhythmicStaff = "Staff 3"
                {
                    \context Voice = "Voice 3"
                    {
                    }
                }
                \context RhythmicStaff = "Staff 4"
                {
                    \context Voice = "Voice 4"
                    {
                    }
                }
            >>
        >>

    ..  container:: example

        More than one voice per staff:

        >>> template_2 = class_(staff_count=[2, 1, 2])
        >>> abjad.show(template_2) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(template_2.__illustrate__()[abjad.Score])
            \context Score = "Grouped Rhythmic Staves Score"
            <<
                \context StaffGroup = "Grouped Rhythmic Staves Staff Group"
                <<
                    \context RhythmicStaff = "Staff 1"
                    <<
                        \context Voice = "Voice 1-1"
                        {
                            s1
                        }
                        \context Voice = "Voice 1-2"
                        {
                            s1
                        }
                    >>
                    \context RhythmicStaff = "Staff 2"
                    {
                        \context Voice = "Voice 2"
                        {
                            s1
                        }
                    }
                    \context RhythmicStaff = "Staff 3"
                    <<
                        \context Voice = "Voice 3-1"
                        {
                            s1
                        }
                        \context Voice = "Voice 3-2"
                        {
                            s1
                        }
                    >>
                >>
            >>

        >>> score = template_2()
        >>> abjad.show(score) # doctest: +SKIP

        >>> abjad.f(score)
        \context Score = "Grouped Rhythmic Staves Score"
        <<
            \context StaffGroup = "Grouped Rhythmic Staves Staff Group"
            <<
                \context RhythmicStaff = "Staff 1"
                <<
                    \context Voice = "Voice 1-1"
                    {
                    }
                    \context Voice = "Voice 1-2"
                    {
                    }
                >>
                \context RhythmicStaff = "Staff 2"
                {
                    \context Voice = "Voice 2"
                    {
                    }
                }
                \context RhythmicStaff = "Staff 3"
                <<
                    \context Voice = "Voice 3-1"
                    {
                    }
                    \context Voice = "Voice 3-2"
                    {
                    }
                >>
            >>
        >>

    """

    ### CLASS VARIABLES ###

    __slots__ = (
        '_staff_count',
        )

    ### INITIALIZER ###

    def __init__(self, staff_count=2):
        super().__init__()
        assert isinstance(staff_count, (int, collections.Iterable))
        if isinstance(staff_count, collections.Iterable):
            staff_count = list(staff_count)
        self._staff_count = staff_count

    ### SPECIAL METHODS ###

    def __call__(self):
        """
        Calls score template.

        Returns score.
        """
        import abjad
        staves = []
        if isinstance(self.staff_count, int):
            for index in range(self.staff_count):
                number = index + 1
                name = 'Voice {}'.format(number)
                voice = abjad.Voice([], name=name)
                name = 'Staff {}'.format(number)
                staff = abjad.Staff([voice], name=name)
                staff.lilypond_type = 'RhythmicStaff'
                abjad.annotate(staff, 'default_clef', abjad.Clef('percussion'))
                staves.append(staff)
                key = 'v{}'.format(number)
                self.voice_abbreviations[key] = voice.name
        elif isinstance(self.staff_count, list):
            for staff_index, voice_count in enumerate(self.staff_count):
                staff_number = staff_index + 1
                name = 'Staff {}'.format(staff_number)
                staff = abjad.Staff(name=name)
                staff.lilypond_type = 'RhythmicStaff'
                assert 1 <= voice_count
                for voice_index in range(voice_count):
                    voice_number = voice_index + 1
                    if voice_count == 1:
                        voice_identifier = str(staff_number)
                    else:
                        voice_identifier = '{}-{}'.format(
                            staff_number, voice_number)
                        staff.is_simultaneous = True
                    name = 'Voice {}'.format(voice_identifier)
                    voice = abjad.Voice([], name=name)
                    staff.append(voice)
                    key = 'v{}'.format(voice_identifier)
                    self.voice_abbreviations[key] = voice.name
                staves.append(staff)
        grouped_rhythmic_staves_staff_group = abjad.StaffGroup(
            staves,
            name='Grouped Rhythmic Staves Staff Group',
            )
        grouped_rhythmic_staves_score = abjad.Score(
            [grouped_rhythmic_staves_staff_group],
            name='Grouped Rhythmic Staves Score',
            )
        return grouped_rhythmic_staves_score

    ### PUBLIC PROPERTIES ###

    @property
    def voice_abbreviations(self):
        """
        Gets context name abbreviations.

        ..  container:: example

            >>> class_ = abjad.GroupedRhythmicStavesScoreTemplate
            >>> template = class_(staff_count=4)
            >>> template.voice_abbreviations
            OrderedDict([])

        """
        return super(
            GroupedRhythmicStavesScoreTemplate,
            self,
            ).voice_abbreviations

    @property
    def staff_count(self):
        """
        Gets score template staff count.

        ..  container:: example

            >>> class_ = abjad.GroupedRhythmicStavesScoreTemplate
            >>> template = class_(staff_count=4)
            >>> template.staff_count
            4

        Returns nonnegative integer.
        """
        return self._staff_count



from abjad.utilities.OrderedDict import OrderedDict
from .Part import Part
from .PartManifest import PartManifest
from .ScoreTemplate import ScoreTemplate


class StringQuartetScoreTemplate(ScoreTemplate):
    r"""
    String quartet score template.

    ..  container:: example

        >>> template = abjad.StringQuartetScoreTemplate()
        >>> abjad.show(template) # doctest: +SKIP

        >>> abjad.f(template.__illustrate__()[abjad.Score])
        \context Score = "String Quartet Score"
        <<
            \context StaffGroup = "String Quartet Staff Group"
            <<
                \tag #'first-violin
                \context Staff = "First Violin Staff"
                {
                    \context Voice = "First Violin Voice"
                    {
                        \clef "treble" %! ST3
                        s1
                    }
                }
                \tag #'second-violin
                \context Staff = "Second Violin Staff"
                {
                    \context Voice = "Second Violin Voice"
                    {
                        \clef "treble" %! ST3
                        s1
                    }
                }
                \tag #'viola
                \context Staff = "Viola Staff"
                {
                    \context Voice = "Viola Voice"
                    {
                        \clef "alto" %! ST3
                        s1
                    }
                }
                \tag #'cello
                \context Staff = "Cello Staff"
                {
                    \context Voice = "Cello Voice"
                    {
                        \clef "bass" %! ST3
                        s1
                    }
                }
            >>
        >>

    Returns score template.
    """

    ### CLASS VARIABLES ###

    __slots__ = ()


    _part_manifest = PartManifest(
        Part(section='FirstViolin', section_abbreviation='VN-1'),
        Part(section='SecondViolin', section_abbreviation='VN-2'),
        Part(section='Viola', section_abbreviation='VA'),
        Part(section='Cello', section_abbreviation='VC'),
        )

    ### INITIALIZER ###

    def __init__(self):
        super().__init__()
        self.voice_abbreviations.update({
            'vn1': 'First Violin Voice',
            'vn2': 'Second Violin Voice',
            'va': 'Viola Voice',
            'vc': 'Cello Voice',
            })

    ### SPECIAL METHODS ###

    def __call__(self):
        """
        Calls string quartet score template.

        Returns score.
        """
        import abjad

        # make first violin voice and staff
        first_violin_voice = abjad.Voice(
            [],
            name='First Violin Voice',
            )
        first_violin_staff = abjad.Staff(
            [first_violin_voice],
            name='First Violin Staff',
            )
        clef = abjad.Clef('treble')
        abjad.annotate(first_violin_staff, 'default_clef', clef)
        violin = abjad.Violin()
        abjad.annotate(first_violin_staff, 'default_instrument', violin)
        tag = abjad.LilyPondLiteral(r"\tag #'first-violin", 'before')
        abjad.attach(tag, first_violin_staff)

        # make second violin voice and staff
        second_violin_voice = abjad.Voice(
            [],
            name='Second Violin Voice',
            )
        second_violin_staff = abjad.Staff(
            [second_violin_voice],
            name='Second Violin Staff',
            )
        clef = abjad.Clef('treble')
        abjad.annotate(second_violin_staff, 'default_clef', clef)
        violin = abjad.Violin()
        abjad.annotate(second_violin_staff, 'default_instrument', violin)
        tag = abjad.LilyPondLiteral(r"\tag #'second-violin", 'before')
        abjad.attach(tag, second_violin_staff)

        # make viola voice and staff
        viola_voice = abjad.Voice(
            [],
            name='Viola Voice',
            )
        viola_staff = abjad.Staff(
            [viola_voice],
            name='Viola Staff',
            )
        clef = abjad.Clef('alto')
        abjad.annotate(viola_staff, 'default_clef', clef)
        viola = abjad.Viola()
        abjad.annotate(viola_staff, 'default_instrument', viola)
        tag = abjad.LilyPondLiteral(r"\tag #'viola", 'before')
        abjad.attach(tag, viola_staff)

        # make cello voice and staff
        cello_voice = abjad.Voice(
            [],
            name='Cello Voice',
            )
        cello_staff = abjad.Staff(
            [cello_voice],
            name='Cello Staff',
            )
        clef = abjad.Clef('bass')
        abjad.annotate(cello_staff, 'default_clef', clef)
        cello = abjad.Cello()
        abjad.annotate(cello_staff, 'default_instrument', cello)
        tag = abjad.LilyPondLiteral(r"\tag #'cello", 'before')
        abjad.attach(tag, cello_staff)

        # make string quartet staff group
        string_quartet_staff_group = abjad.StaffGroup([
            first_violin_staff,
            second_violin_staff,
            viola_staff,
            cello_staff,
            ],
            name='String Quartet Staff Group',
            )

        # make string quartet score
        string_quartet_score = abjad.Score(
            [string_quartet_staff_group],
            name='String Quartet Score',
            )

        # return string quartet score
        return string_quartet_score



import abjad
import {score_package_name}


class SegmentMaker(abjad.AbjadObject):
    """
    Segment-maker.
    """

    ### INITIALIZER ###

    def __init__(self):
        self.score_template = {score_package_name}.ScoreTemplate()

    ### PUBLIC METHODS ###

    def run(
        self,
        metadata=None,
        midi=None,
        previous_metadata=None,
        ):
        """
        Runs segment-maker.

        Returns LilyPond file.
        """
        self.metadata = metadata
        self.midi = midi
        self.previous_metadata = previous_metadata
        score = self.score_template()
        score['Example Voice'].extend("c'4 ( d'4 e'4 f'4 )")
        lilypond_file = abjad.LilyPondFile.new(
            score,
            includes=['../../stylesheets/stylesheet.ily'],
            )
        return lilypond_file



from abjad.system.AbjadObject import AbjadObject


class LilyPondDuration(AbjadObject):
    """
    Model of a duration in LilyPond.

    Not composer-safe.

    Used internally by LilyPondParser.
    """

    ### CLASS VARIABLES ###

    __slots__ = (
        'duration',
        'multiplier',
        )

    ### INITIALIZER ###

    def __init__(self, duration=None, multiplier=None):
        self.duration = duration
        self.multiplier = multiplier



import abc
import collections
import types
from abjad import mathtools
from abjad.system.FormatSpecification import FormatSpecification
from abjad.utilities.TypedCollection import TypedCollection
from abjad.utilities.TypedCounter import TypedCounter


class Vector(TypedCounter):
    """
    Abstract vector.
    """

    ### CLASS VARIABLES ##

    __slots__ = ()

    ### INITIALIZER ###

    def __init__(self, items=None, item_class=None):
        prototype_1 = (collections.Iterator, types.GeneratorType)
        prototype_2 = (TypedCounter, collections.Counter)
        if isinstance(items, str):
            items = items.split()
        elif isinstance(items, prototype_1):
            items = [item for item in items]
        elif isinstance(items, dict):
            items = self._dictionary_to_items(items, item_class)
        if isinstance(items, prototype_2):
            new_tokens = []
            for item, count in items.items():
                new_tokens.extend(count * [item])
            items = new_tokens
        if item_class is None:
            item_class = self._named_item_class
            if items is not None:
                if (isinstance(items, TypedCollection) and
                    issubclass(items.item_class, self._parent_item_class)):
                    item_class = items.item_class
                elif len(items):
                    if isinstance(items, collections.Set):
                        items = tuple(items)
                    if isinstance(items, dict):
                        item_class = self._dictionary_to_item_class(items)
                    elif isinstance(items[0], str):
                        item_class = self._named_item_class
                    elif isinstance(items[0], (int, float)):
                        item_class = self._numbered_item_class
                    elif isinstance(items[0], self._parent_item_class):
                        item_class = type(items[0])
        assert issubclass(item_class, self._parent_item_class)
        TypedCounter.__init__(
            self,
            items=items,
            item_class=item_class,
            )

    ### SPECIAL METHODS ###

    def __str__(self):
        """
        String representation of vector.

        Returns string.
        """
        parts = ['{}: {}'.format(key, value)
            for key, value in self.items()]
        return '<{}>'.format(', '.join(parts))

    ### PRIVATE PROPERTIES ###

    @abc.abstractproperty
    def _named_item_class(self):
        raise NotImplementedError

    @abc.abstractproperty
    def _numbered_item_class(self):
        raise NotImplementedError

    @abc.abstractproperty
    def _parent_item_class(self):
        raise NotImplementedError

    ### PRIVATE METHODS ###

    def _dictionary_to_item_class(self, dictionary):
        if not len(dictionary):
            return self._named_item_class
        keys = dictionary.keys()
        first_key = keys[0]
        assert isinstance(first_key, str), repr(first_key)
        try:
            float(first_key)
            item_class = self._numbered_item_class
        except ValueError:
            item_class = self._named_item_class
        return item_class

    def _dictionary_to_items(self, dictionary, item_class):
        items = []
        for initializer_token, count in dictionary.items():
            for _ in range(count):
                item = item_class(initializer_token)
                items.append(item)
        return items

    def _get_format_specification(self):
        if self.item_class.__name__.startswith('Named'):
            repr_items = {str(k): v for k, v in self.items()}
        else:
            repr_items = {
                mathtools.integer_equivalent_number_to_integer(
                    float(k.number)): v
                for k, v in self.items()
                }
        return FormatSpecification(
            client=self,
            repr_is_indented=False,
            repr_args_values=[repr_items],
            storage_format_args_values=[self._collection],
            )

    ### PUBLIC METHODS ###

    @abc.abstractmethod
    def from_selection(
        class_,
        selection,
        item_class=None,
        ):
        """
        Makes vector from `selection`.

        Returns vector.
        """
        raise NotImplementedError



#! /usr/bin/env python
import abjad
import sys


class FerneyhoughDemo:
    r"""
    Ferneyhough demo.

    ..  container:: example

        Initializes Ferneyhough demo:

        >>> demo = abjad.demos.ferneyhough.FerneyhoughDemo()

        Calls Ferneyhough demo:

        >>> lilypond_file = demo(
        ...     tuplet_duration=abjad.Duration(1, 4),
        ...     row_count=11,
        ...     column_count=6,
        ...     )

        Shows LilyPond file:

        >>> abjad.show(lilypond_file) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(lilypond_file[abjad.Score])
            \new Score \with {
                \override BarLine.stencil = ##f
                \override BarNumber.transparent = ##t
                \override SpacingSpanner.strict-note-spacing = ##t
                \override SpacingSpanner.uniform-stretching = ##t
                \override TimeSignature.stencil = ##f
                \override TupletBracket.padding = #2
                \override TupletBracket.staff-padding = #4
                \override TupletNumber.text = #tuplet-number::calc-fraction-text
                proportionalNotationDuration = #(ly:make-moment 1 56)
                tupletFullLength = ##t
            } <<
                \new RhythmicStaff {
                    \tweak text #tuplet-number::calc-fraction-text
                    \times 1/1 {
                        \time 1/4
                        c'8
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 1/1 {
                            c'8
                        }
                    }
                    \tweak text #tuplet-number::calc-fraction-text
                    \times 1/1 {
                        c'8
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 1/1 {
                            c'16
                            c'16
                        }
                    }
                    \tweak text #tuplet-number::calc-fraction-text
                    \times 1/1 {
                        c'8
                        \times 2/3 {
                            c'16
                            c'16
                            c'16
                        }
                    }
                    \tweak text #tuplet-number::calc-fraction-text
                    \times 1/1 {
                        c'8
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 1/1 {
                            c'32
                            c'32
                            c'32
                            c'32
                        }
                    }
                    \tweak text #tuplet-number::calc-fraction-text
                    \times 1/1 {
                        c'8
                        \times 4/5 {
                            c'32
                            c'32
                            c'32
                            c'32
                            c'32
                        }
                    }
                    \tweak text #tuplet-number::calc-fraction-text
                    \times 1/1 {
                        c'8
                        \times 2/3 {
                            c'32
                            c'32
                            c'32
                            c'32
                            c'32
                            c'32
                        }
                    }
                }
                \new RhythmicStaff {
                    \times 2/3 {
                        \time 1/4
                        c'8
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 1/1 {
                            c'4
                        }
                    }
                    \times 2/3 {
                        c'8
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 1/1 {
                            c'8
                            c'8
                        }
                    }
                    \times 2/3 {
                        c'8
                        \times 2/3 {
                            c'8
                            c'8
                            c'8
                        }
                    }
                    \times 2/3 {
                        c'8
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 1/1 {
                            c'16
                            c'16
                            c'16
                            c'16
                        }
                    }
                    \times 2/3 {
                        c'8
                        \times 4/5 {
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                        }
                    }
                    \times 2/3 {
                        c'8
                        \times 2/3 {
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                        }
                    }
                }
                \new RhythmicStaff {
                    \tweak text #tuplet-number::calc-fraction-text
                    \times 1/1 {
                        \time 1/4
                        c'16
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 3/4 {
                            c'4
                        }
                    }
                    \tweak text #tuplet-number::calc-fraction-text
                    \times 1/1 {
                        c'16
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 3/4 {
                            c'8
                            c'8
                        }
                    }
                    \tweak text #tuplet-number::calc-fraction-text
                    \times 1/1 {
                        c'16
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 1/1 {
                            c'16
                            c'16
                            c'16
                        }
                    }
                    \tweak text #tuplet-number::calc-fraction-text
                    \times 1/1 {
                        c'16
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 3/4 {
                            c'16
                            c'16
                            c'16
                            c'16
                        }
                    }
                    \tweak text #tuplet-number::calc-fraction-text
                    \times 1/1 {
                        c'16
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 3/5 {
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                        }
                    }
                    \tweak text #tuplet-number::calc-fraction-text
                    \times 1/1 {
                        c'16
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 1/1 {
                            c'32
                            c'32
                            c'32
                            c'32
                            c'32
                            c'32
                        }
                    }
                }
                \new RhythmicStaff {
                    \times 4/5 {
                        \time 1/4
                        c'16
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 1/1 {
                            c'4
                        }
                    }
                    \times 4/5 {
                        c'16
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 1/1 {
                            c'8
                            c'8
                        }
                    }
                    \times 4/5 {
                        c'16
                        \times 2/3 {
                            c'8
                            c'8
                            c'8
                        }
                    }
                    \times 4/5 {
                        c'16
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 1/1 {
                            c'16
                            c'16
                            c'16
                            c'16
                        }
                    }
                    \times 4/5 {
                        c'16
                        \times 4/5 {
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                        }
                    }
                    \times 4/5 {
                        c'16
                        \times 2/3 {
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                        }
                    }
                }
                \new RhythmicStaff {
                    \times 2/3 {
                        \time 1/4
                        c'16
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 5/8 {
                            c'2
                        }
                    }
                    \times 2/3 {
                        c'16
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 5/8 {
                            c'4
                            c'4
                        }
                    }
                    \times 2/3 {
                        c'16
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 5/6 {
                            c'8
                            c'8
                            c'8
                        }
                    }
                    \times 2/3 {
                        c'16
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 5/8 {
                            c'8
                            c'8
                            c'8
                            c'8
                        }
                    }
                    \times 2/3 {
                        c'16
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 1/1 {
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                        }
                    }
                    \times 2/3 {
                        c'16
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 5/6 {
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                        }
                    }
                }
                \new RhythmicStaff {
                    \times 4/7 {
                        \time 1/4
                        c'16
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 3/4 {
                            c'2
                        }
                    }
                    \times 4/7 {
                        c'16
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 3/4 {
                            c'4
                            c'4
                        }
                    }
                    \times 4/7 {
                        c'16
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 1/1 {
                            c'8
                            c'8
                            c'8
                        }
                    }
                    \times 4/7 {
                        c'16
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 3/4 {
                            c'8
                            c'8
                            c'8
                            c'8
                        }
                    }
                    \times 4/7 {
                        c'16
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 3/5 {
                            c'8
                            c'8
                            c'8
                            c'8
                            c'8
                        }
                    }
                    \times 4/7 {
                        c'16
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 1/1 {
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                        }
                    }
                }
                \new RhythmicStaff {
                    \tweak text #tuplet-number::calc-fraction-text
                    \times 1/1 {
                        \time 1/4
                        c'32
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 7/8 {
                            c'4
                        }
                    }
                    \tweak text #tuplet-number::calc-fraction-text
                    \times 1/1 {
                        c'32
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 7/8 {
                            c'8
                            c'8
                        }
                    }
                    \tweak text #tuplet-number::calc-fraction-text
                    \times 1/1 {
                        c'32
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 7/12 {
                            c'8
                            c'8
                            c'8
                        }
                    }
                    \tweak text #tuplet-number::calc-fraction-text
                    \times 1/1 {
                        c'32
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 7/8 {
                            c'16
                            c'16
                            c'16
                            c'16
                        }
                    }
                    \tweak text #tuplet-number::calc-fraction-text
                    \times 1/1 {
                        c'32
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 7/10 {
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                        }
                    }
                    \tweak text #tuplet-number::calc-fraction-text
                    \times 1/1 {
                        c'32
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 7/12 {
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                        }
                    }
                }
                \new RhythmicStaff {
                    \times 8/9 {
                        \time 1/4
                        c'32
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 1/1 {
                            c'4
                        }
                    }
                    \times 8/9 {
                        c'32
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 1/1 {
                            c'8
                            c'8
                        }
                    }
                    \times 8/9 {
                        c'32
                        \times 2/3 {
                            c'8
                            c'8
                            c'8
                        }
                    }
                    \times 8/9 {
                        c'32
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 1/1 {
                            c'16
                            c'16
                            c'16
                            c'16
                        }
                    }
                    \times 8/9 {
                        c'32
                        \times 4/5 {
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                        }
                    }
                    \times 8/9 {
                        c'32
                        \times 2/3 {
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                        }
                    }
                }
                \new RhythmicStaff {
                    \times 4/5 {
                        \time 1/4
                        c'32
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 9/16 {
                            c'2
                        }
                    }
                    \times 4/5 {
                        c'32
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 9/16 {
                            c'4
                            c'4
                        }
                    }
                    \times 4/5 {
                        c'32
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 3/4 {
                            c'8
                            c'8
                            c'8
                        }
                    }
                    \times 4/5 {
                        c'32
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 9/16 {
                            c'8
                            c'8
                            c'8
                            c'8
                        }
                    }
                    \times 4/5 {
                        c'32
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 9/10 {
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                        }
                    }
                    \times 4/5 {
                        c'32
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 3/4 {
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                        }
                    }
                }
                \new RhythmicStaff {
                    \times 8/11 {
                        \time 1/4
                        c'32
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 5/8 {
                            c'2
                        }
                    }
                    \times 8/11 {
                        c'32
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 5/8 {
                            c'4
                            c'4
                        }
                    }
                    \times 8/11 {
                        c'32
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 5/6 {
                            c'8
                            c'8
                            c'8
                        }
                    }
                    \times 8/11 {
                        c'32
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 5/8 {
                            c'8
                            c'8
                            c'8
                            c'8
                        }
                    }
                    \times 8/11 {
                        c'32
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 1/1 {
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                        }
                    }
                    \times 8/11 {
                        c'32
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 5/6 {
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                        }
                    }
                }
                \new RhythmicStaff {
                    \times 2/3 {
                        \time 1/4
                        c'32
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 11/16 {
                            c'2
                        }
                    }
                    \times 2/3 {
                        c'32
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 11/16 {
                            c'4
                            c'4
                        }
                    }
                    \times 2/3 {
                        c'32
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 11/12 {
                            c'8
                            c'8
                            c'8
                        }
                    }
                    \times 2/3 {
                        c'32
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 11/16 {
                            c'8
                            c'8
                            c'8
                            c'8
                        }
                    }
                    \times 2/3 {
                        c'32
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 11/20 {
                            c'8
                            c'8
                            c'8
                            c'8
                            c'8
                        }
                    }
                    \times 2/3 {
                        c'32
                        \tweak text #tuplet-number::calc-fraction-text
                        \times 11/12 {
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                        }
                    }
                }
            >>

    """

    ### SPECIAL METHODS ###

    def __call__(self, tuplet_duration=(1, 4), row_count=11, column_count=6):
        """
        Calls Ferneyhough demo.

        Returns LilyPond file.
        """
        lilypond_file = self.make_lilypond_file(
            tuplet_duration,
            row_count,
            column_count,
            )
        return lilypond_file

    ### PUBLIC METHODS ###

    def configure_lilypond_file(self, lilypond_file):
        """
        Configures LilyPond file.
        """
        lilypond_file._default_paper_size = '11x17', 'portrait'
        lilypond_file._global_staff_size = 12
        lilypond_file.layout_block.indent = 0
        lilypond_file.layout_block.ragged_right = True
        lilypond_file.paper_block.ragged_bottom = True
        spacing_vector = abjad.SpacingVector(0, 0, 8, 0)
        lilypond_file.paper_block.system_system_spacing = spacing_vector

    def configure_score(self, score):
        """
        Configures ``score``.
        """
        moment = abjad.SchemeMoment((1, 56))
        abjad.setting(score).proportional_notation_duration = moment
        abjad.setting(score).tuplet_full_length = True
        abjad.override(score).bar_line.stencil = False
        abjad.override(score).bar_number.transparent = True
        abjad.override(score).spacing_spanner.uniform_stretching = True
        abjad.override(score).spacing_spanner.strict_note_spacing = True
        abjad.override(score).time_signature.stencil = False
        abjad.override(score).tuplet_bracket.padding = 2
        abjad.override(score).tuplet_bracket.staff_padding = 4
        scheme = abjad.Scheme('tuplet-number::calc-fraction-text')
        abjad.override(score).tuplet_number.text = scheme

    def make_lilypond_file(self, tuplet_duration, row_count, column_count):
        """
        Makes LilyPond file.
        """
        score = self.make_score(
            tuplet_duration,
            row_count,
            column_count,
            )
        self.configure_score(score)
        lilypond_file = abjad.LilyPondFile.new(score)
        self.configure_lilypond_file(lilypond_file)
        return lilypond_file

    def make_nested_tuplet(
        self,
        tuplet_duration,
        outer_tuplet_proportions,
        inner_tuplet_subdivision_count,
    ):
        """
        Makes nested tuplet.
        """
        outer_tuplet = abjad.Tuplet.from_duration_and_ratio(
            tuplet_duration,
            outer_tuplet_proportions,
            )
        inner_tuplet_proportions = inner_tuplet_subdivision_count * [1]
        selector = abjad.select().leaves()
        last_leaf = selector(outer_tuplet)[-1]
        right_logical_tie = abjad.inspect(last_leaf).logical_tie()
        right_logical_tie.to_tuplet(inner_tuplet_proportions)
        return outer_tuplet

    def make_row_of_nested_tuplets(
        self,
        tuplet_duration,
        outer_tuplet_proportions,
        column_count,
    ):
        """
        Makes row of nested tuplets.
        """
        assert 0 < column_count
        row_of_nested_tuplets = []
        for n in range(column_count):
            inner_tuplet_subdivision_count = n + 1
            nested_tuplet = self.make_nested_tuplet(
                tuplet_duration,
                outer_tuplet_proportions,
                inner_tuplet_subdivision_count,
                )
            row_of_nested_tuplets.append(nested_tuplet)
        return row_of_nested_tuplets

    def make_rows_of_nested_tuplets(
        self,
        tuplet_duration,
        row_count,
        column_count,
    ):
        """
        Makes rows of nested tuplets.
        """
        assert 0 < row_count
        rows_of_nested_tuplets = []
        for n in range(row_count):
            outer_tuplet_proportions = (1, n + 1)
            row_of_nested_tuplets = self.make_row_of_nested_tuplets(
                tuplet_duration,
                outer_tuplet_proportions,
                column_count,
                )
            rows_of_nested_tuplets.append(row_of_nested_tuplets)
        return rows_of_nested_tuplets

    def make_score(self, tuplet_duration, row_count, column_count):
        """
        Makes score.
        """
        score = abjad.Score()
        rows_of_nested_tuplets = self.make_rows_of_nested_tuplets(
            tuplet_duration,
            row_count,
            column_count,
            )
        for row_of_nested_tuplets in rows_of_nested_tuplets:
            staff = abjad.Staff(row_of_nested_tuplets)
            staff.lilypond_type = 'RhythmicStaff'
            time_signature = abjad.TimeSignature((1, 4))
            leaf = abjad.inspect(staff).leaf(0)
            abjad.attach(time_signature, leaf)
            score.append(staff)
        return score


if __name__ == '__main__':
    from abjad import show

    arguments = sys.argv[1:]
    if len(arguments) != 3:
        print('USAGE: tuplet_duration row_count column_count')
        print(' e.g.: python main.py 1/4 11 6')
        sys.exit(0)

    tuplet_duration = abjad.Duration(arguments[0])
    row_count = int(arguments[1])
    column_count = int(arguments[2])

    assert 0 < tuplet_duration
    assert 0 < row_count
    assert 0 < column_count

    demo = abjad.demos.ferneyhough.FerneyhoughDemo()
    lilypond_file = demo(
        tuplet_duration=tuplet_duration,
        row_count=row_count,
        column_count=column_count,
        )
    show(lilypond_file)



from abjad import mathtools
from abjad.system.FormatSpecification import FormatSpecification
from .IntervalClass import IntervalClass


class NumberedIntervalClass(IntervalClass):
    """
    Numbered interval-class.

    ..  container:: example

        Initializes from integer:

        >>> abjad.NumberedIntervalClass(-14)
        NumberedIntervalClass(-2)

    ..  container:: example

        Initializes from float:

        >>> abjad.NumberedIntervalClass(-14.5)
        NumberedIntervalClass(-2.5)

    ..  container:: example

        Initializes from string:

        >>> abjad.NumberedIntervalClass('-14.5')
        NumberedIntervalClass(-2.5)

        >>> abjad.NumberedIntervalClass('P8')
        NumberedIntervalClass(12)

        >>> abjad.NumberedIntervalClass('-P8')
        NumberedIntervalClass(-12)

    """

    ### CLASS VARIABLES ###

    __slots__ = (
        '_number',
        )

    ### INITIALIZER ###

    def __init__(self, number=0):
        super().__init__(number or 0)

    ### SPECIAL METHODS ###

    def __abs__(self):
        """
        Gets absolute value of numbered interval-class.

        Returns new numbered interval-class.
        """
        return type(self)(abs(self.number))

    def __add__(self, argument):
        """
        Adds `argument` to numbered interval-class.

        Returns new numbered interval-class.
        """
        try:
            argument = type(self)(argument)
        except Exception:
            return NotImplemented
        return type(self)(float(self) + float(argument))

    def __radd__(self, argument):
        """
        Adds `argument` to numbered interval-class.

        Returns new numbered interval-class.
        """
        try:
            argument = type(self)(argument)
        except Exception:
            return NotImplemented
        return type(self)(float(self) + float(argument))

    def __eq__(self, argument):
        """
        Is true when `argument` is a numbered interval-class with number
        equal to that of this numbered interval-class.

        ..  container:: example

            >>> interval_class_1 = abjad.NumberedIntervalClass(0)
            >>> interval_class_2 = abjad.NumberedIntervalClass(0)
            >>> interval_class_3 = abjad.NumberedIntervalClass(1)

            >>> interval_class_1 == interval_class_1
            True
            >>> interval_class_1 == interval_class_2
            True
            >>> interval_class_1 == interval_class_3
            False

            >>> interval_class_2 == interval_class_1
            True
            >>> interval_class_2 == interval_class_2
            True
            >>> interval_class_2 == interval_class_3
            False

            >>> interval_class_3 == interval_class_1
            False
            >>> interval_class_3 == interval_class_2
            False
            >>> interval_class_3 == interval_class_3
            True

        Returns true or false.
        """
        return super().__eq__(argument)

    def __float__(self):
        """
        Coerce to semitones as float.

        Returns float.
        """
        return float(self._number)

    def __hash__(self):
        """
        Hashes numbered interval-class.

        Returns integer.
        """
        return super().__hash__()

    def __lt__(self, argument):
        """
        Is true when numbered interval-class is less than `argument`.

        ..  container:: example

            >>> interval_class_1 = abjad.NumberedIntervalClass(0)
            >>> interval_class_2 = abjad.NumberedIntervalClass(0)
            >>> interval_class_3 = abjad.NumberedIntervalClass(1)

            >>> interval_class_1 < interval_class_1
            False
            >>> interval_class_1 < interval_class_2
            False
            >>> interval_class_1 < interval_class_3
            True

            >>> interval_class_2 < interval_class_1
            False
            >>> interval_class_2 < interval_class_2
            False
            >>> interval_class_2 < interval_class_3
            True

            >>> interval_class_3 < interval_class_1
            False
            >>> interval_class_3 < interval_class_2
            False
            >>> interval_class_3 < interval_class_3
            False

        Returns true or false.
        """
        try:
            argument = type(self)(argument)
        except Exception:
            return False
        return self.number < argument.number

    def __str__(self):
        """
        Gets string representation of numbered interval-class.

        ..  container:: example

            >>> str(abjad.NumberedIntervalClass(-13))
            '-1'

            >>> str(abjad.NumberedIntervalClass(0))
            '0'

            >>> str(abjad.NumberedIntervalClass(13))
            '+1'

        """
        string = super().__str__()
        if 0 < self.number:
            string = '+' + string
        return string

    def __sub__(self, argument):
        """
        Subtracts `argument` from numbered interval-class.

        Returns new numbered interval-class.
        """
        try:
            argument = type(self)(argument)
        except Exception:
            return NotImplemented
        return type(self)(float(self) - float(argument))

    ### PRIVATE METHODS ###

    def _from_named_parts(self, direction, quality, diatonic_number):
        self._number = self._named_to_numbered(
            direction,
            quality,
            diatonic_number,
            )

    def _from_number(self, argument):
        direction = mathtools.sign(argument)
        number = self._to_nearest_quarter_tone(abs(argument))
        pc_number = number % 12
        if pc_number == 0 and number:
            pc_number = 12
        self._number = pc_number * direction

    def _from_interval_or_interval_class(self, argument):
        self._from_number(float(argument))

    def _get_format_specification(self):
        return FormatSpecification(
            client=self,
            coerce_for_equality=True,
            repr_is_indented=False,
            storage_format_is_indented=False,
            storage_format_args_values=[self.number],
            )

    ### PUBLIC PROPERTIES ###

    @property
    def direction_number(self):
        """
        Gets direction number of numbered interval-class.

        Returns -1, 0 or 1.
        """
        if self.number < 1:
            return -1
        elif self.number == 1:
            return 0
        else:
            return 1

    ### PUBLIC METHODS ###

    @classmethod
    def from_pitch_carriers(class_, pitch_carrier_1, pitch_carrier_2):
        """Makes numbered interval-class from `pitch_carrier_1` and
        `pitch_carrier_2`.

        ..  container:: example

            >>> abjad.NumberedIntervalClass.from_pitch_carriers(
            ...     abjad.NamedPitch(-2),
            ...     abjad.NamedPitch(12),
            ...     )
            NumberedIntervalClass(2)

            >>> abjad.NumberedIntervalClass.from_pitch_carriers(
            ...     abjad.NamedPitch(0),
            ...     abjad.NamedPitch(12),
            ...     )
            NumberedIntervalClass(12)

            >>> abjad.NumberedIntervalClass.from_pitch_carriers(
            ...     abjad.NamedPitch(9),
            ...     abjad.NamedPitch(12),
            ...     )
            NumberedIntervalClass(3)

            >>> abjad.NumberedIntervalClass.from_pitch_carriers(
            ...     abjad.NamedPitch(12),
            ...     abjad.NamedPitch(9),
            ...     )
            NumberedIntervalClass(-3)

            >>> abjad.NumberedIntervalClass.from_pitch_carriers(
            ...     abjad.NamedPitch(12),
            ...     abjad.NamedPitch(12),
            ...     )
            NumberedIntervalClass(0)

            >>> abjad.NumberedIntervalClass.from_pitch_carriers(
            ...     abjad.NamedPitch(24),
            ...     abjad.NamedPitch(0),
            ...     )
            NumberedIntervalClass(-12)

            >>> abjad.NumberedIntervalClass.from_pitch_carriers(
            ...     abjad.NamedPitch(12),
            ...     abjad.NamedPitch(-2),
            ...     )
            NumberedIntervalClass(-2)

        Returns numbered interval-class.
        """
        import abjad
        interval = abjad.NumberedInterval.from_pitch_carriers(
            pitch_carrier_1,
            pitch_carrier_2,
            )
        return class_(interval)



"""
Tools for modeling and manipulating 24ET pitches.
"""
from .constants import *
from .Accidental import Accidental
from .ColorMap import ColorMap
from .CompoundOperator import CompoundOperator
from .Duplication import Duplication
from .Interval import Interval
from .IntervalClass import IntervalClass
from .Segment import Segment
from .IntervalClassSegment import IntervalClassSegment
from .Set import Set
from .IntervalClassSet import IntervalClassSet
from .Vector import Vector
from .IntervalClassVector import IntervalClassVector
from .IntervalSegment import IntervalSegment
from .IntervalSet import IntervalSet
from .IntervalVector import IntervalVector
from .Inversion import Inversion
from .Multiplication import Multiplication
from .NamedInterval import NamedInterval
from .NamedIntervalClass import NamedIntervalClass
from .NamedInversionEquivalentIntervalClass import NamedInversionEquivalentIntervalClass
from .Octave import Octave
from .PitchClass import PitchClass
from .Pitch import Pitch
from .NamedPitch import NamedPitch
from .NamedPitchClass import NamedPitchClass
from .NumberedInterval import NumberedInterval
from .NumberedIntervalClass import NumberedIntervalClass
from .NumberedInversionEquivalentIntervalClass import NumberedInversionEquivalentIntervalClass
from .NumberedPitch import NumberedPitch
from .NumberedPitchClass import NumberedPitchClass
from .PitchClassSegment import PitchClassSegment
from .PitchClassSet import PitchClassSet
from .PitchClassVector import PitchClassVector
from .PitchRange import PitchRange
from .PitchSegment import PitchSegment
from .PitchSet import PitchSet
from .PitchVector import PitchVector
from .Retrograde import Retrograde
from .Rotation import Rotation
from .SetClass import SetClass
from .StaffPosition import StaffPosition
from .Transposition import Transposition
from .TwelveToneRow import TwelveToneRow



import collections
import copy
import inspect
import pathlib
from abjad.indicators.LilyPondLiteral import LilyPondLiteral
from abjad.system.AbjadObject import AbjadObject
from abjad.pitch.NamedPitch import NamedPitch
from abjad.core.Component import Component
from abjad.core.Container import Container
from abjad.core.Context import Context
from abjad.core.MeasureMaker import MeasureMaker
from abjad.core.Note import Note
from abjad.core.Score import Score
from abjad.core.Selection import Selection
from abjad.core.Staff import Staff
from abjad.core.Voice import Voice
from abjad.scheme import Scheme
from abjad.scheme import SpacingVector
from abjad.system.StorageFormatManager import StorageFormatManager
from abjad.top.attach import attach
from abjad.top.inspect import inspect as abjad_inspect
from abjad.top.iterate import iterate
from abjad.top.mutate import mutate
from abjad.top.override import override
from abjad.top.sequence import sequence
from .Block import Block
from .ContextBlock import ContextBlock
from .DateTimeToken import DateTimeToken
from .LilyPondLanguageToken import LilyPondLanguageToken
from .LilyPondVersionToken import LilyPondVersionToken


class LilyPondFile(AbjadObject):
    r"""
    A LilyPond file.

    ..  container:: example

        >>> staff = abjad.Staff("c'8 d'8 e'8 f'8")
        >>> comments = [
        ...     'File construct as an example.',
        ...     'Parts shown here for positioning.',
        ...     ]
        >>> includes = [
        ...     'external-settings-file-1.ily',
        ...     'external-settings-file-2.ily',
        ...     ]
        >>> lilypond_file = abjad.LilyPondFile.new(
        ...     music=staff,
        ...     default_paper_size=('a5', 'portrait'),
        ...     comments=comments,
        ...     includes=includes,
        ...     global_staff_size=16,
        ...     )

        >>> lilypond_file.header_block.composer = abjad.Markup('Josquin')
        >>> lilypond_file.header_block.title = abjad.Markup('Missa sexti tonus')
        >>> lilypond_file.layout_block.indent = 0
        >>> lilypond_file.layout_block.left_margin = 15
        >>> abjad.show(lilypond_file) # doctest: +SKIP

        ::

            >>> print(format(lilypond_file)) # doctest: +SKIP
            % 2004-01-14 17:29

            % File construct as an example.
            % Parts shown here for positioning.

            \version "2.19.0"
            \language "english"

            \include "external-settings-file-1.ily"
            \include "external-settings-file-2.ily"

            #(set-default-paper-size "a5" 'portrait)
            #(set-global-staff-size 16)

            \header {
                composer = \markup { Josquin }
                title = \markup { Missa sexti toni }
            }

            \layout {
                indent = #0
                left-margin = #15
            }

            \paper {
            }

            \new Staff {
                c'8
                d'8
                e'8
                f'8
            }

    """

    ### CLASS VARIABLES ###

    __slots__ = (
        '_comments',
        '_date_time_token',
        '_default_paper_size',
        '_global_staff_size',
        '_includes',
        '_items',
        '_lilypond_language_token',
        '_lilypond_version_token',
        '_use_relative_includes',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        comments=None,
        date_time_token=None,
        default_paper_size=None,
        global_staff_size=None,
        includes=None,
        items=None,
        lilypond_language_token=None,
        lilypond_version_token=None,
        use_relative_includes=None,
        ):
        comments = comments or ()
        comments = tuple(comments)
        self._comments = comments
        self._date_time_token = None
        if bool(date_time_token):
            self._date_time_token = DateTimeToken()
        self._default_paper_size = default_paper_size
        self._global_staff_size = global_staff_size
        includes = list(includes or [])
        includes = [str(_) for _ in includes]
        self._includes = includes
        self._items = list(items or [])
        self._lilypond_language_token = None
        if lilypond_language_token is not False:
            token = LilyPondLanguageToken()
            self._lilypond_language_token = token
        self._lilypond_version_token = None
        if lilypond_version_token is not False:
            token = LilyPondVersionToken()
            self._lilypond_version_token = token
        self._use_relative_includes = use_relative_includes

    ### SPECIAL METHODS ###

    def __contains__(self, argument) -> bool:
        """
        Is true when LilyPond file contains ``argument``.

        ..  container:: example

            >>> staff = abjad.Staff("c'4 d' e' f'")
            >>> lilypond_file = abjad.LilyPondFile.new(staff)

            >>> staff in lilypond_file
            True

            >>> abjad.Staff in lilypond_file
            True

            >>> 'Allegro' in lilypond_file
            False

            >>> 0 in lilypond_file
            False

        """
        try:
            item = self[argument]
            return True
        except (AssertionError, KeyError, ValueError, TypeError):
            return False
        
    def __format__(self, format_specification=''):
        r"""
        Formats LilyPond file.

        ..  container:: example

            Gets format:

            >>> lilypond_file = abjad.LilyPondFile.new()

            >>> print(format(lilypond_file)) # doctest: +SKIP
            % 2016-01-31 20:29
            <BLANKLINE>
            \version "2.19.35"
            \language "english"
            <BLANKLINE>
            \header {}
            <BLANKLINE>
            \layout {}
            <BLANKLINE>
            \paper {}

        ..  container:: example

            Works with empty layout and MIDI blocks:

            >>> score = abjad.Score([abjad.Staff("c'8 d'8 e'8 f'8")])
            >>> score_block = abjad.Block(name='score')
            >>> layout_block = abjad.Block(name='layout')
            >>> midi_block = abjad.Block(name='midi')
            >>> score_block.items.append(score)
            >>> score_block.items.append(layout_block)
            >>> score_block.items.append(midi_block)

            >>> abjad.f(score_block)
            \score {
                \new Score
                <<
                    \new Staff
                    {
                        c'8
                        d'8
                        e'8
                        f'8
                    }
                >>
                \layout {}
                \midi {}
            }

        ..  container:: example

            >>> staff = abjad.Staff("c'8 d' e' f'")
            >>> abjad.attach(abjad.Articulation('.'), staff[0])
            >>> abjad.attach(abjad.Markup('Allegro'), staff[0])
            >>> score = abjad.Score([staff])
            >>> lilypond_file = abjad.LilyPondFile.new([score])
            >>> lilypond_file._lilypond_version_token = None

            >>> abjad.f(lilypond_file)
            \language "english"
            <BLANKLINE>
            \header {
                tagline = ##f
            }
            <BLANKLINE>
            \layout {}
            <BLANKLINE>
            \paper {}
            <BLANKLINE>
            \score {
                {
                    \new Score
                    <<
                        \new Staff
                        {
                            c'8
                            -\staccato
                            - \markup { Allegro }
                            d'8
                            e'8
                            f'8
                        }
                    >>
                }
            }

        Returns string.
        """
        if format_specification in ('', 'lilypond'):
            return self._get_lilypond_format()
        else:
            assert format_specification == 'storage'
            return StorageFormatManager(self).get_storage_format()

    def __getitem__(self, name):
        r"""
        Gets item with ``name``.

        ..  container:: example

            Gets header block:

            >>> lilypond_file = abjad.LilyPondFile.new()

            >>> lilypond_file['header']
            <Block(name='header')>

        ..  container:: example

            Searches score:

            >>> voice_1 = abjad.Voice("c''4 b' a' g'", name='Custom Voice 1')
            >>> abjad.attach(abjad.LilyPondLiteral(r'\voiceOne'), voice_1)
            >>> voice_2 = abjad.Voice("c'4 d' e' f'", name='Custom Voice 2')
            >>> abjad.attach(abjad.LilyPondLiteral(r'\voiceTwo'), voice_2)
            >>> staff = abjad.Staff(
            ...     [voice_1, voice_2],
            ...     is_simultaneous=True,
            ...     name='Custom Staff',
            ...     )
            >>> score = abjad.Score([staff], name='Custom Score')
            >>> lilypond_file = abjad.LilyPondFile.new(score)
            >>> abjad.show(score) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(score)
                \context Score = "Custom Score"
                <<
                    \context Staff = "Custom Staff"
                    <<
                        \context Voice = "Custom Voice 1"
                        {
                            \voiceOne
                            c''4
                            b'4
                            a'4
                            g'4
                        }
                        \context Voice = "Custom Voice 2"
                        {
                            \voiceTwo
                            c'4
                            d'4
                            e'4
                            f'4
                        }
                    >>
                >>

            >>> lilypond_file['score']
            <Block(name='score')>

            >>> lilypond_file['Custom Score']
            <Score-"Custom Score"<<1>>>

            >>> lilypond_file[abjad.Score]
            <Score-"Custom Score"<<1>>>

            >>> lilypond_file['Custom Staff']
            <Staff-"Custom Staff"<<2>>>

            >>> lilypond_file[abjad.Staff]
            <Staff-"Custom Staff"<<2>>>

            >>> lilypond_file['Custom Voice 1']
            Voice("c''4 b'4 a'4 g'4", name='Custom Voice 1')

            >>> lilypond_file['Custom Voice 2']
            Voice("c'4 d'4 e'4 f'4", name='Custom Voice 2')

            >>> lilypond_file[abjad.Voice]
            Voice("c''4 b'4 a'4 g'4", name='Custom Voice 1')

        ..  container:: example

            REGRESSION. Works when score block contains parallel container:

                >>> include_container = abjad.Container()
                >>> string = r'\include "layout.ly"'
                >>> literal = abjad.LilyPondLiteral(string, 'opening')
                >>> abjad.attach(literal, include_container)
                >>> staff = abjad.Staff("c'4 d' e' f'", name='CustomStaff')
                >>> container = abjad.Container(
                ...     [include_container, staff],
                ...     is_simultaneous=True,
                ...     )
                >>> lilypond_file = abjad.LilyPondFile.new(
                ...     container,
                ...     lilypond_language_token=False,
                ...     lilypond_version_token=False,
                ...     )
                >>> del(lilypond_file.items[:3])

                >>> abjad.f(lilypond_file)
                \score {
                    <<
                        {
                            \include "layout.ly"
                        }
                        \context Staff = "CustomStaff"
                        {
                            c'4
                            d'4
                            e'4
                            f'4
                        }
                    >>
                }

                >>> lilypond_file[abjad.Staff]
                Staff("c'4 d'4 e'4 f'4", name='CustomStaff')

                >>> lilypond_file['CustomStaff']
                Staff("c'4 d'4 e'4 f'4", name='CustomStaff')

        Returns item.

        Raises key error when no item with ``name`` is found.
        """
        if not isinstance(name, str):
            if inspect.isclass(name):
                assert issubclass(name, Component), repr(name)
            else:
                assert isinstance(name, Component), repr(name)
        score = None
        if self.score_block and self.score_block.items:
            items = self.score_block.items
            for container in iterate(items).components(Container):
                if isinstance(container, Context):
                    score = container
                    break
        if isinstance(name, str):
            for item in self.items:
                if getattr(item, 'name', None) == name:
                    return item
            if score is not None:
                if score.name == name:
                    return score
                context = score[name]
                return context
            raise KeyError(f'can not find item with name {name!r}.')
        elif isinstance(name, Component):
            for item in self.items:
                if item is name:
                    return item
            if score is not None:
                if score is name:
                    return score
                prototype = Context
                for context in iterate(score).components(prototype):
                    if context is name:
                        return context
            raise KeyError(f'can not find {name}.')
        elif inspect.isclass(name) and issubclass(name, Component):
            for item in self.items:
                if isinstance(item, name):
                    return item
            if score is not None:
                if isinstance(score, name):
                    return score
                prototype = Context
                for context in iterate(score).components(prototype):
                    if isinstance(context, name):
                        return context
            raise KeyError(f'can not find item of class {name}.')
        else:
            raise TypeError(name)

    def __illustrate__(self):
        """
        Illustrates LilyPond file.

        Returns LilyPond file unchanged.
        """
        return self

    def __repr__(self):
        """
        Gets interpreter representation of LilyPond file.

        ..  container:: example

            Gets interpreter representation:

            >>> lilypond_file = abjad.LilyPondFile.new()

            >>> lilypond_file
            LilyPondFile(comments=[],
            includes=[],
            items=[<Block(name='header')>, <Block(name='layout')>,
            <Block(name='paper')>, <Block(name='score')>],
            lilypond_language_token=LilyPondLanguageToken(),
            lilypond_version_token=LilyPondVersionToken(version_string='...'))

        Returns string.
        """
        return super().__repr__()

    ### PRIVATE METHODS ###

    def _get_format_pieces(self):
        result = []
        if self.date_time_token is not None:
            string = f'% {self.date_time_token}'
            result.append(string)
        result.extend(self._get_formatted_comments())
        includes = []
        if self.lilypond_version_token is not None:
            string = f'{self.lilypond_version_token}'
            includes.append(string)
        if self.lilypond_language_token is not None:
            string = f'{self.lilypond_language_token}'
            includes.append(string)
        includes = '\n'.join(includes)
        if includes:
            result.append(includes)
        if self.use_relative_includes:
            string = "#(ly:set-option 'relative-includes #t)"
            result.append(string)
        result.extend(self._get_formatted_includes())
        result.extend(self._get_formatted_scheme_settings())
        result.extend(self._get_formatted_blocks())
        return result

    ### PRIVATE METHODS ###

    def _get_formatted_blocks(self):
        result = []
        for item in self.items:
            if ('_get_lilypond_format' in dir(item) and
                not isinstance(item, str)):
                try:
                    string = item._get_lilypond_format()
                except TypeError:
                    string = item._get_lilypond_format()
                if string:
                    result.append(string)
            else:
                result.append(str(item))
        return result

    def _get_formatted_comments(self):
        result = []
        for comment in self.comments:
            if ('_get_lilypond_format' in dir(comment) and
                not isinstance(comment, str)):
                lilypond_format = format(comment)
                if lilypond_format:
                    string = f'% {comment}'
                    result.append(string)
            else:
                string = f'% {comment!s}'
                result.append(string)
        if result:
            result = ['\n'.join(result)]
        return result

    def _get_formatted_includes(self):
        result = []
        for include in self.includes:
            if isinstance(include, str):
                string = rf'\include "{include}"'
                result.append(string)
            elif isinstance(include, pathlib.Path):
                string = rf'\include "{include!s}"'
                result.append(string)
            else:
                result.append(format(include))
        if result:
            result = ['\n'.join(result)]
        return result

    def _get_formatted_scheme_settings(self):
        result = []
        default_paper_size = self.default_paper_size
        if default_paper_size is not None:
            dimension, orientation = default_paper_size
            string = f"#(set-default-paper-size \"{dimension}\" '{orientation})"
            result.append(string)
        global_staff_size = self.global_staff_size
        if global_staff_size is not None:
            string = f'#(set-global-staff-size {global_staff_size})'
            result.append(string)
        if result:
            result = ['\n'.join(result)]
        return result

    def _get_lilypond_format(self):
        return '\n\n'.join(self._get_format_pieces())

    @staticmethod
    def _make_global_context_block(
        font_size=3,
        minimum_distance=10,
        padding=4,
        ):
        assert isinstance(font_size, (int, float))
        assert isinstance(padding, (int, float))
        block = ContextBlock(
            type_='Engraver_group',
            name='GlobalContext',
            )
        block.consists_commands.append('Axis_group_engraver')
        block.consists_commands.append('Time_signature_engraver')
        time_signature_grob = override(block).time_signature
        time_signature_grob.X_extent = (0, 0)
        time_signature_grob.X_offset = Scheme(
            'ly:self-alignment-interface::x-aligned-on-self'
            )
        time_signature_grob.Y_extent = (0, 0)
        time_signature_grob.break_align_symbol = False
        time_signature_grob.break_visibility = Scheme(
            'end-of-line-invisible',
            )
        time_signature_grob.font_size = font_size
        time_signature_grob.self_alignment_X = Scheme('center')
        spacing_vector = SpacingVector(
            0,
            minimum_distance,
            padding,
            0,
            )
        grob = override(block).vertical_axis_group
        grob.default_staff_staff_spacing = spacing_vector
        return block

    ### PUBLIC PROPERTIES ###

    @property
    def comments(self):
        """
        Gets comments of Lilypond file.

        ..  container:: example

            Gets comments:

            >>> lilypond_file = abjad.LilyPondFile.new()

            >>> lilypond_file.comments
            []

        Returns list.
        """
        return list(self._comments)

    @property
    def date_time_token(self):
        """
        Gets date-time token.

        ..  container:: example

            Gets date-time token:

            >>> lilypond_file = abjad.LilyPondFile.new(date_time_token=True)

            >>> lilypond_file.date_time_token
            DateTimeToken()

        Returns date-time token or none.
        """
        return self._date_time_token

    @property
    def default_paper_size(self):
        """
        Gets default paper size of LilyPond file.

        ..  container:: example

            Gets default paper size:

            >>> lilypond_file = abjad.LilyPondFile.new()

            >>> lilypond_file.default_paper_size is None
            True

        Set to pair or none.

        Defaults to none.

        Returns pair or none.
        """
        return self._default_paper_size

    @property
    def global_staff_size(self):
        """
        Gets global staff size of LilyPond file.

        ..  container:: example

            Gets global staff size:

            >>> lilypond_file = abjad.LilyPondFile.new()

            >>> lilypond_file.global_staff_size is None
            True

        Set to number or none.

        Defaults to none.

        Returns number or none.
        """
        return self._global_staff_size

    @property
    def header_block(self):
        """
        Gets header block.

        ..  container:: example

            Gets header block:

            >>> lilypond_file = abjad.LilyPondFile.new()

            >>> lilypond_file.header_block
            <Block(name='header')>

        Returns block or none.
        """
        for item in self.items:
            if isinstance(item, Block):
                if item.name == 'header':
                    return item

    @property
    def includes(self):
        """
        Gets includes of LilyPond file.

        ..  container:: example

            Gets includes:

            >>> lilypond_file = abjad.LilyPondFile.new()

            >>> lilypond_file.includes
            []

        Return list
        """
        return self._includes

    @property
    def items(self):
        r"""
        Gets items in LilyPond file.

        ..  container:: example

            Gets items:

            >>> lilypond_file = abjad.LilyPondFile.new()

            >>> for item in lilypond_file.items:
            ...     item
            ...
            <Block(name='header')>
            <Block(name='layout')>
            <Block(name='paper')>
            <Block(name='score')>

        ..  container:: example

            Accepts strings:

            >>> staff = abjad.Staff("c'4 d' e' f'")
            >>> score_block = abjad.Block(name='score')
            >>> score_block.items.append(staff)
            >>> lilypond_file = abjad.LilyPondFile(
            ...     lilypond_language_token=False,
            ...     lilypond_version_token=False,
            ...     )
            >>> string = r'\customCommand'
            >>> lilypond_file.items.append(string)
            >>> lilypond_file.items.append(score_block)

            >>> abjad.f(lilypond_file)
            \customCommand
            <BLANKLINE>
            \score {
                \new Staff
                {
                    c'4
                    d'4
                    e'4
                    f'4
                }
            }

        ..  container:: example

            Returns list:

            >>> isinstance(lilypond_file.items, list)
            True

        Returns list.
        """
        return self._items

    @property
    def layout_block(self):
        """
        Gets layout block.

        ..  container:: example

            Gets layout block:

            >>> lilypond_file = abjad.LilyPondFile.new()

            >>> lilypond_file.layout_block
            <Block(name='layout')>

        Returns block or none.
        """
        for item in self.items:
            if isinstance(item, Block):
                if item.name == 'layout':
                    return item

    @property
    def lilypond_language_token(self):
        """
        Gets LilyPond language token.

        ..  container:: example

            Gets LilyPond language token:

            >>> lilypond_file = abjad.LilyPondFile.new()

            >>> lilypond_file.lilypond_language_token
            LilyPondLanguageToken()

        Returns LilyPond language token or none.
        """
        return self._lilypond_language_token

    @property
    def lilypond_version_token(self):
        """
        Gets LilyPond version token.

        ..  container:: example

            Gets LilyPond version token:

            >>> lilypond_file = abjad.LilyPondFile.new()

            >>> lilypond_file.lilypond_version_token # doctest: +SKIP
            LilyPondVersionToken('2.19.35')

        Returns LilyPond version token or none.
        """
        return self._lilypond_version_token

    @property
    def paper_block(self):
        """
        Gets paper block.

        ..  container:: example

            Gets paper block:

            >>> lilypond_file = abjad.LilyPondFile.new()

            >>> lilypond_file.paper_block
            <Block(name='paper')>

        Returns block or none.
        """
        for item in self.items:
            if isinstance(item, Block):
                if item.name == 'paper':
                    return item

    @property
    def score_block(self):
        """
        Gets score block.

        ..  container:: example

            Gets score block:

            >>> lilypond_file = abjad.LilyPondFile.new()

            >>> lilypond_file.score_block
            <Block(name='score')>

        Returns block or none.
        """
        for item in self.items:
            if isinstance(item, Block):
                if item.name == 'score':
                    return item

    @property
    def use_relative_includes(self):
        """
        Is true when LilyPond file should use relative includes.

        ..  container:: example

            Gets relative include flag:

            >>> lilypond_file = abjad.LilyPondFile.new()

            >>> lilypond_file.use_relative_includes is None
            True

        Set to true, false or none.

        Returns true, false or none.
        """
        return self._use_relative_includes

    ### PUBLIC METHODS ###

    @classmethod
    def new(
        class_,
        music=None,
        date_time_token=None,
        default_paper_size=None,
        comments=None,
        includes=None,
        global_staff_size=None,
        lilypond_language_token=None,
        lilypond_version_token=None,
        use_relative_includes=None,
        ):
        r"""
        Makes basic LilyPond file.

        ..  container:: example

            Makes basic LilyPond file:

            >>> score = abjad.Score([abjad.Staff("c'8 d'8 e'8 f'8")])
            >>> lilypond_file = abjad.LilyPondFile.new(score)
            >>> lilypond_file.header_block.title = abjad.Markup('Missa sexti tonus')
            >>> lilypond_file.header_block.composer = abjad.Markup('Josquin')
            >>> lilypond_file.layout_block.indent = 0
            >>> lilypond_file.paper_block.top_margin = 15
            >>> lilypond_file.paper_block.left_margin = 15

            ::

                >>> abjad.f(lilypond_file) # doctest: +SKIP
                \header {
                    composer = \markup { Josquin }
                    title = \markup { Missa sexti tonus }
                }

                \layout {
                    indent = #0
                }

                \paper {
                    left-margin = #15
                    top-margin = #15
                }

                \score {
                    \new Score <<
                        \new Staff {
                            c'8
                            d'8
                            e'8
                            f'8
                        }
                    >>
                }

            >>> abjad.show(lilypond_file) # doctest: +SKIP

        Wraps ``music`` in LilyPond ``\score`` block.

        Adds LilyPond ``\header``, ``\layout``, ``\paper`` and ``\score``
        blocks to LilyPond file.

        Returns LilyPond file.
        """
        if isinstance(music, LilyPondFile):
            return music
        lilypond_file = class_(
            date_time_token=date_time_token,
            default_paper_size=default_paper_size,
            comments=comments,
            includes=includes,
            items=[
                Block(name='header'),
                Block(name='layout'),
                Block(name='paper'),
                Block(name='score'),
                ],
            global_staff_size=global_staff_size,
            lilypond_language_token=lilypond_language_token,
            lilypond_version_token=lilypond_version_token,
            use_relative_includes=use_relative_includes,
            )
        lilypond_file.header_block.tagline = False
        if music is not None:
            lilypond_file.score_block.items.append(music)
        return lilypond_file

    @classmethod
    def rhythm(
        class_,
        selections,
        divisions=None,
        attach_lilypond_voice_commands=None,
        implicit_scaling=None,
        pitched_staff=None,
        simultaneous_selections=None,
        time_signatures=None,
        ):
        r"""
        Makes rhythm-styled LilyPond file.

        ..  container:: example

            Makes rhythmic staff:

            >>> divisions = [(3, 4), (4, 8), (1, 4)]
            >>> maker = abjad.NoteMaker()
            >>> selections = [
            ...     maker(6 * [0], [(1, 8)]),
            ...     maker(8 * [0], [(1, 16)]),
            ...     maker(2 * [0], [(1, 8)]),
            ...     ]
            >>> for selection in selections:
            ...     abjad.attach(abjad.Beam(), selection[:])
            ...
            >>> lilypond_file = abjad.LilyPondFile.rhythm(
            ...     selections,
            ...     divisions,
            ...     )
            >>> abjad.show(lilypond_file) # doctest: +SKIP

            ..  docs::

                >>> score = lilypond_file[abjad.Score]
                >>> abjad.f(score)
                \new Score
                <<
                    \new GlobalContext
                    {
                        {   % measure
                            \time 3/4
                            s1 * 3/4
                        }   % measure
                        {   % measure
                            \time 4/8
                            s1 * 1/2
                        }   % measure
                        {   % measure
                            \time 1/4
                            s1 * 1/4
                        }   % measure
                    }
                    \new RhythmicStaff
                    {
                        {   % measure
                            \time 3/4
                            c'8
                            [
                            c'8
                            c'8
                            c'8
                            c'8
                            c'8
                            ]
                        }   % measure
                        {   % measure
                            \time 4/8
                            c'16
                            [
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            ]
                        }   % measure
                        {   % measure
                            \time 1/4
                            c'8
                            [
                            c'8
                            ]
                        }   % measure
                    }
                >>

        ..  container:: example

            Set time signatures explicitly:

            >>> divisions = [(3, 4), (4, 8), (1, 4)]
            >>> maker = abjad.NoteMaker()
            >>> selections = [
            ...     maker(6 * [0], [(1, 8)]),
            ...     maker(8 * [0], [(1, 16)]),
            ...     maker(2 * [0], [(1, 8)]),
            ...     ]
            >>> for selection in selections:
            ...     abjad.attach(abjad.Beam(), selection[:])
            ...
            >>> lilypond_file = abjad.LilyPondFile.rhythm(
            ...     selections,
            ...     [(6, 8), (4, 8), (2, 8)],
            ...     )
            >>> abjad.show(lilypond_file) # doctest: +SKIP

            ..  docs::

                >>> score = lilypond_file[abjad.Score]
                >>> abjad.f(score)
                \new Score
                <<
                    \new GlobalContext
                    {
                        {   % measure
                            \time 6/8
                            s1 * 3/4
                        }   % measure
                        {   % measure
                            \time 4/8
                            s1 * 1/2
                        }   % measure
                        {   % measure
                            \time 2/8
                            s1 * 1/4
                        }   % measure
                    }
                    \new RhythmicStaff
                    {
                        {   % measure
                            \time 6/8
                            c'8
                            [
                            c'8
                            c'8
                            c'8
                            c'8
                            c'8
                            ]
                        }   % measure
                        {   % measure
                            \time 4/8
                            c'16
                            [
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            ]
                        }   % measure
                        {   % measure
                            \time 2/8
                            c'8
                            [
                            c'8
                            ]
                        }   % measure
                    }
                >>

        ..  container:: example

            Makes pitched staff:

            >>> divisions = [(3, 4), (4, 8), (1, 4)]
            >>> maker = abjad.NoteMaker()
            >>> selections = [
            ...     maker(6 * [0], [(1, 8)]),
            ...     maker(8 * [0], [(1, 16)]),
            ...     maker(2 * [0], [(1, 8)]),
            ...     ]
            >>> for selection in selections:
            ...     abjad.attach(abjad.Beam(), selection[:])
            ...
            >>> lilypond_file = abjad.LilyPondFile.rhythm(
            ...     selections,
            ...     divisions,
            ...     pitched_staff=True,
            ...     )
            >>> abjad.show(lilypond_file) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(lilypond_file[abjad.Score])
                \new Score
                <<
                    \new GlobalContext
                    {
                        {   % measure
                            \time 3/4
                            s1 * 3/4
                        }   % measure
                        {   % measure
                            \time 4/8
                            s1 * 1/2
                        }   % measure
                        {   % measure
                            \time 1/4
                            s1 * 1/4
                        }   % measure
                    }
                    \new Staff
                    {
                        {   % measure
                            \time 3/4
                            c'8
                            [
                            c'8
                            c'8
                            c'8
                            c'8
                            c'8
                            ]
                        }   % measure
                        {   % measure
                            \time 4/8
                            c'16
                            [
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            ]
                        }   % measure
                        {   % measure
                            \time 1/4
                            c'8
                            [
                            c'8
                            ]
                        }   % measure
                    }
                >>

        ..  container:: example

            Makes simultaneous voices:

            >>> divisions = [(3, 4), (4, 8), (1, 4)]
            >>> maker = abjad.NoteMaker()
            >>> selections = [
            ...     maker(6 * [0], [(1, 8)]),
            ...     maker(8 * [0], [(1, 16)]),
            ...     maker(2 * [0], [(1, 8)]),
            ...     ]
            >>> for selection in selections:
            ...     abjad.attach(abjad.Beam(), selection[:])
            ...
            >>> for note in abjad.iterate(selections).components(abjad.Note):
            ...     note.written_pitch = abjad.NamedPitch("e'")
            ...
            >>> selection_1 = selections[0] + selections[1] + selections[2]
            >>> selections = [
            ...     maker(12 * [0], [(1, 16)]),
            ...     maker(16 * [0], [(1, 32)]),
            ...     maker(4 * [0], [(1, 16)]),
            ...     ]
            >>> for selection in selections:
            ...     abjad.attach(abjad.Beam(), selection[:])
            ...
            >>> selection_2 = selections[0] + selections[1] + selections[2]
            >>> selections = {
            ...     'Voice 1': selection_1,
            ...     'Voice 2': selection_2,
            ... }
            >>> lilypond_file = abjad.LilyPondFile.rhythm(
            ...     selections,
            ...     divisions,
            ...     )
            >>> voice_1 = lilypond_file['Voice 1']
            >>> abjad.attach(abjad.LilyPondLiteral(r'\voiceOne'), voice_1)
            >>> voice_2 = lilypond_file['Voice 2']
            >>> abjad.attach(abjad.LilyPondLiteral(r'\voiceTwo'), voice_2)
            >>> abjad.show(lilypond_file) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(lilypond_file[abjad.Score])
                \new Score
                <<
                    \new GlobalContext
                    {
                        {   % measure
                            \time 3/4
                            s1 * 3/4
                        }   % measure
                        {   % measure
                            \time 4/8
                            s1 * 1/2
                        }   % measure
                        {   % measure
                            \time 1/4
                            s1 * 1/4
                        }   % measure
                    }
                    \new Staff
                    <<
                        \context Voice = "Voice 1"
                        {
                            \voiceOne
                            e'8
                            [
                            e'8
                            e'8
                            e'8
                            e'8
                            e'8
                            ]
                            e'16
                            [
                            e'16
                            e'16
                            e'16
                            e'16
                            e'16
                            e'16
                            e'16
                            ]
                            e'8
                            [
                            e'8
                            ]
                        }
                        \context Voice = "Voice 2"
                        {
                            \voiceTwo
                            c'16
                            [
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            c'16
                            ]
                            c'32
                            [
                            c'32
                            c'32
                            c'32
                            c'32
                            c'32
                            c'32
                            c'32
                            c'32
                            c'32
                            c'32
                            c'32
                            c'32
                            c'32
                            c'32
                            c'32
                            ]
                            c'16
                            [
                            c'16
                            c'16
                            c'16
                            ]
                        }
                    >>
                >>

        Returns LilyPond file.
        """
        if isinstance(selections, list):
            for selection in selections:
                if not isinstance(selection, Selection):
                    message = f'must be selection: {selection!r}.'
                    raise TypeError(message)
        elif isinstance(selections, dict):
            for selection in selections.values():
                if not isinstance(selection, Selection):
                    message = f'must be selection: {selection!r}.'
                    raise TypeError(message)
        else:
            message = f'must be list or dictionary: {selections!r}.'
            raise TypeError(message)
        score = Score()
        lilypond_file = LilyPondFile.new(
            score,
            includes=['default.ily', 'rhythm-maker-docs.ily'],
            )
        if pitched_staff is None:
            if isinstance(selections, list):
                selections_ = selections
            elif isinstance(selections, dict):
                selections_ = selections.values()
            else:
                raise TypeError(selections)
            for note in iterate(selections_).leaves(Note):
                if note.written_pitch != NamedPitch("c'"):
                    pitched_staff = True
                    break
        if isinstance(selections, list):
            if divisions is None:
                duration = abjad_inspect(selections).duration()
                divisions = [duration]
            time_signatures = time_signatures or divisions
            maker = MeasureMaker(implicit_scaling=implicit_scaling)
            measures = maker(time_signatures)
            if pitched_staff:
                staff = Staff(measures)
            else:
                staff = Staff(measures, lilypond_type='RhythmicStaff')
            selections = sequence(selections).flatten(depth=-1)
            selections_ = copy.deepcopy(selections)
            try:
                agent = mutate(staff)
                measures = agent.replace_measure_contents(selections)
            except StopIteration:
                if pitched_staff:
                    staff = Staff(selections_)
                else:
                    staff = Staff(
                        selections_,
                        lilypond_type='RhythmicStaff',
                        )
        elif isinstance(selections, dict):
            voices = []
            for voice_name in sorted(selections):
                selections_ = selections[voice_name]
                selections_ = sequence(selections_).flatten(depth=-1)
                selections_ = copy.deepcopy(selections_)
                voice = Voice(selections_, name=voice_name)
                if attach_lilypond_voice_commands:
                    voice_name_to_command_string = {
                        'Voice 1': 'voiceOne',
                        'Voice 2': 'voiceTwo',
                        'Voice 3': 'voiceThree',
                        'Voice 4': 'voiceFour',
                        }
                    command_string = voice_name_to_command_string.get(
                        voice_name,
                        )
                    if command_string:
                        command = LilyPondLiteral('\\' + command_string)
                        attach(command, voice)
                voices.append(voice)
            staff = Staff(voices, is_simultaneous=True)
            if divisions is None:
                duration = abjad_inspect(staff).duration()
                divisions = [duration]
        else:
            message = 'must be list or dictionary of selections:'
            message += f' {selections!r}.'
            raise TypeError(message)
        score.append(staff)
        assert isinstance(divisions, collections.Sequence), repr(divisions)
        time_signatures = time_signatures or divisions
        context = Context(lilypond_type='GlobalContext')
        maker = MeasureMaker(implicit_scaling=implicit_scaling)
        measures = maker(time_signatures)
        context.extend(measures)
        score.insert(0, context)
        return lilypond_file



import typing
from abjad.system.AbjadValueObject import AbjadValueObject
from .LilyPondGrobInterface import LilyPondGrobInterface


class LilyPondGrob(AbjadValueObject):
    """
    LilyPond grob.

    ..  container:: example

        >>> grob = abjad.lilypondnames.LilyPondGrob('Beam')
        >>> print(format(grob))
        abjad.LilyPondGrob(
            name='Beam',
            )

    """

    ### CLASS VARIABLES ###

    __slots__ = (
        '_name',
        )

    _identity_map: typing.Dict[str, 'LilyPondGrob'] = {}

    ### CONSTRUCTOR ###

    def __new__(class_, name='NoteHead'):
        if name in class_._identity_map:
            obj = class_._identity_map[name]
        else:
            obj = object.__new__(class_)
            class_._identity_map[name] = obj
        return obj

    ### INITIALIZER ###

    def __init__(self, name='NoteHead') -> None:
        from abjad.ly import grob_interfaces
        assert name in grob_interfaces
        self._name = name

    ### PUBLIC PROPERTIES ###

    @property
    def interfaces(self) -> typing.Tuple[LilyPondGrobInterface, ...]:
        """
        Gets interfaces of LilyPond grob.

        ..  container:: example

            >>> grob = abjad.lilypondnames.LilyPondGrob('Beam')
            >>> for interface in grob.interfaces:
            ...     interface
            ...
            LilyPondGrobInterface(name='beam-interface')
            LilyPondGrobInterface(name='font-interface')
            LilyPondGrobInterface(name='grob-interface')
            LilyPondGrobInterface(name='spanner-interface')
            LilyPondGrobInterface(name='staff-symbol-referencer-interface')
            LilyPondGrobInterface(name='unbreakable-spanner-interface')

        """
        from abjad.ly import grob_interfaces
        return tuple(
            LilyPondGrobInterface(_)
            for _ in sorted(grob_interfaces[self.name])
            )

    @property
    def name(self) -> str:
        """
        Gets name of LilyPond grob.

        ..  container:: example

            >>> grob = abjad.lilypondnames.LilyPondGrob('Beam')
            >>> grob.name
            'Beam'

        """
        return self._name

    @property
    def property_names(self) -> typing.Tuple[str, ...]:
        """
        Gets property names of LilyPond grob.

        ..  container:: example

            >>> grob = abjad.lilypondnames.LilyPondGrob('Beam')
            >>> for property_name in grob.property_names:
            ...     property_name
            ...
            'X-extent'
            'X-offset'
            'X-positions'
            'Y-extent'
            'Y-offset'
            'after-line-breaking'
            'annotation'
            'auto-knee-gap'
            'avoid-slur'
            'beam-thickness'
            'beamed-stem-shorten'
            'beaming'
            'before-line-breaking'
            'break-overshoot'
            'breakable'
            'clip-edges'
            'collision-interfaces'
            'collision-voice-only'
            'color'
            'concaveness'
            'cross-staff'
            'damping'
            'details'
            'direction'
            'extra-offset'
            'font-encoding'
            'font-family'
            'font-name'
            'font-series'
            'font-shape'
            'font-size'
            'footnote-music'
            'forced-spacing'
            'gap'
            'gap-count'
            'grow-direction'
            'horizontal-skylines'
            'id'
            'inspect-quants'
            'knee'
            'layer'
            'length-fraction'
            'minimum-X-extent'
            'minimum-Y-extent'
            'minimum-length'
            'minimum-length-after-break'
            'neutral-direction'
            'normalized-endpoints'
            'parenthesis-friends'
            'positions'
            'rotation'
            'skip-quanting'
            'skyline-horizontal-padding'
            'spanner-id'
            'springs-and-rods'
            'staff-position'
            'stencil'
            'to-barline'
            'transparent'
            'vertical-skylines'
            'whiteout'
            'whiteout-box'

        """
        property_names: typing.Set[str] = set()
        for interface in self.interfaces:
            property_names.update(interface.property_names)
        return tuple(sorted(property_names))

    ### PUBLIC METHODS ###

    @staticmethod
    def list_all_grobs() -> typing.Tuple['LilyPondGrob', ...]:
        """
        Lists all grobs.

        ..  container:: example

            >>> for lilypond_grob in abjad.lilypondnames.LilyPondGrob.list_all_grobs():
            ...     lilypond_grob
            ...
            LilyPondGrob(name='Accidental')
            LilyPondGrob(name='AccidentalCautionary')
            LilyPondGrob(name='AccidentalPlacement')
            LilyPondGrob(name='AccidentalSuggestion')
            LilyPondGrob(name='Ambitus')
            LilyPondGrob(name='AmbitusAccidental')
            LilyPondGrob(name='AmbitusLine')
            LilyPondGrob(name='AmbitusNoteHead')
            LilyPondGrob(name='Arpeggio')
            LilyPondGrob(name='BalloonTextItem')
            LilyPondGrob(name='BarLine')
            LilyPondGrob(name='BarNumber')
            LilyPondGrob(name='BassFigure')
            LilyPondGrob(name='BassFigureAlignment')
            LilyPondGrob(name='BassFigureAlignmentPositioning')
            LilyPondGrob(name='BassFigureBracket')
            LilyPondGrob(name='BassFigureContinuation')
            LilyPondGrob(name='BassFigureLine')
            LilyPondGrob(name='Beam')
            LilyPondGrob(name='BendAfter')
            LilyPondGrob(name='BreakAlignGroup')
            LilyPondGrob(name='BreakAlignment')
            LilyPondGrob(name='BreathingSign')
            LilyPondGrob(name='ChordName')
            LilyPondGrob(name='Clef')
            LilyPondGrob(name='ClefModifier')
            LilyPondGrob(name='ClusterSpanner')
            LilyPondGrob(name='ClusterSpannerBeacon')
            LilyPondGrob(name='CombineTextScript')
            LilyPondGrob(name='CueClef')
            LilyPondGrob(name='CueEndClef')
            LilyPondGrob(name='Custos')
            LilyPondGrob(name='DotColumn')
            LilyPondGrob(name='Dots')
            LilyPondGrob(name='DoublePercentRepeat')
            LilyPondGrob(name='DoublePercentRepeatCounter')
            LilyPondGrob(name='DoubleRepeatSlash')
            LilyPondGrob(name='DynamicLineSpanner')
            LilyPondGrob(name='DynamicText')
            LilyPondGrob(name='DynamicTextSpanner')
            LilyPondGrob(name='Episema')
            LilyPondGrob(name='Fingering')
            LilyPondGrob(name='FingeringColumn')
            LilyPondGrob(name='Flag')
            LilyPondGrob(name='FootnoteItem')
            LilyPondGrob(name='FootnoteSpanner')
            LilyPondGrob(name='FretBoard')
            LilyPondGrob(name='Glissando')
            LilyPondGrob(name='GraceSpacing')
            LilyPondGrob(name='GridLine')
            LilyPondGrob(name='GridPoint')
            LilyPondGrob(name='Hairpin')
            LilyPondGrob(name='HorizontalBracket')
            LilyPondGrob(name='InstrumentName')
            LilyPondGrob(name='InstrumentSwitch')
            LilyPondGrob(name='KeyCancellation')
            LilyPondGrob(name='KeySignature')
            LilyPondGrob(name='KievanLigature')
            LilyPondGrob(name='LaissezVibrerTie')
            LilyPondGrob(name='LaissezVibrerTieColumn')
            LilyPondGrob(name='LedgerLineSpanner')
            LilyPondGrob(name='LeftEdge')
            LilyPondGrob(name='LigatureBracket')
            LilyPondGrob(name='LyricExtender')
            LilyPondGrob(name='LyricHyphen')
            LilyPondGrob(name='LyricSpace')
            LilyPondGrob(name='LyricText')
            LilyPondGrob(name='MeasureCounter')
            LilyPondGrob(name='MeasureGrouping')
            LilyPondGrob(name='MelodyItem')
            LilyPondGrob(name='MensuralLigature')
            LilyPondGrob(name='MetronomeMark')
            LilyPondGrob(name='MultiMeasureRest')
            LilyPondGrob(name='MultiMeasureRestNumber')
            LilyPondGrob(name='MultiMeasureRestText')
            LilyPondGrob(name='NonMusicalPaperColumn')
            LilyPondGrob(name='NoteCollision')
            LilyPondGrob(name='NoteColumn')
            LilyPondGrob(name='NoteHead')
            LilyPondGrob(name='NoteName')
            LilyPondGrob(name='NoteSpacing')
            LilyPondGrob(name='OttavaBracket')
            LilyPondGrob(name='PaperColumn')
            LilyPondGrob(name='ParenthesesItem')
            LilyPondGrob(name='PercentRepeat')
            LilyPondGrob(name='PercentRepeatCounter')
            LilyPondGrob(name='PhrasingSlur')
            LilyPondGrob(name='PianoPedalBracket')
            LilyPondGrob(name='RehearsalMark')
            LilyPondGrob(name='RepeatSlash')
            LilyPondGrob(name='RepeatTie')
            LilyPondGrob(name='RepeatTieColumn')
            LilyPondGrob(name='Rest')
            LilyPondGrob(name='RestCollision')
            LilyPondGrob(name='Script')
            LilyPondGrob(name='ScriptColumn')
            LilyPondGrob(name='ScriptRow')
            LilyPondGrob(name='Slur')
            LilyPondGrob(name='SostenutoPedal')
            LilyPondGrob(name='SostenutoPedalLineSpanner')
            LilyPondGrob(name='SpacingSpanner')
            LilyPondGrob(name='SpanBar')
            LilyPondGrob(name='SpanBarStub')
            LilyPondGrob(name='StaffGrouper')
            LilyPondGrob(name='StaffSpacing')
            LilyPondGrob(name='StaffSymbol')
            LilyPondGrob(name='StanzaNumber')
            LilyPondGrob(name='Stem')
            LilyPondGrob(name='StemStub')
            LilyPondGrob(name='StemTremolo')
            LilyPondGrob(name='StringNumber')
            LilyPondGrob(name='StrokeFinger')
            LilyPondGrob(name='SustainPedal')
            LilyPondGrob(name='SustainPedalLineSpanner')
            LilyPondGrob(name='System')
            LilyPondGrob(name='SystemStartBar')
            LilyPondGrob(name='SystemStartBrace')
            LilyPondGrob(name='SystemStartBracket')
            LilyPondGrob(name='SystemStartSquare')
            LilyPondGrob(name='TabNoteHead')
            LilyPondGrob(name='TextScript')
            LilyPondGrob(name='TextSpanner')
            LilyPondGrob(name='Tie')
            LilyPondGrob(name='TieColumn')
            LilyPondGrob(name='TimeSignature')
            LilyPondGrob(name='TrillPitchAccidental')
            LilyPondGrob(name='TrillPitchGroup')
            LilyPondGrob(name='TrillPitchHead')
            LilyPondGrob(name='TrillSpanner')
            LilyPondGrob(name='TupletBracket')
            LilyPondGrob(name='TupletNumber')
            LilyPondGrob(name='UnaCordaPedal')
            LilyPondGrob(name='UnaCordaPedalLineSpanner')
            LilyPondGrob(name='VaticanaLigature')
            LilyPondGrob(name='VerticalAlignment')
            LilyPondGrob(name='VerticalAxisGroup')
            LilyPondGrob(name='VoiceFollower')
            LilyPondGrob(name='VoltaBracket')
            LilyPondGrob(name='VoltaBracketSpanner')

        """
        from abjad.ly import grob_interfaces
        return tuple(LilyPondGrob(name) for name in sorted(grob_interfaces))



import typing
from abjad import enums
from abjad.system.AbjadValueObject import AbjadValueObject
from abjad.lilypondnames.LilyPondTweakManager import LilyPondTweakManager
from abjad.system.LilyPondFormatBundle import LilyPondFormatBundle


class BreathMark(AbjadValueObject):
    r"""
    Breath mark.

    ..  container:: example

        Attached to a single note:

        >>> note = abjad.Note("c'4")
        >>> breath_mark = abjad.BreathMark()
        >>> abjad.attach(breath_mark, note)
        >>> abjad.show(note) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(note)
            c'4
            \breathe

    ..  container:: example

        Attached to notes in a staff:

        >>> staff = abjad.Staff("c'8 d' e' f' g' a' b' c''")
        >>> abjad.attach(abjad.Beam(), staff[:4])
        >>> abjad.attach(abjad.Beam(), staff[4:])
        >>> abjad.attach(abjad.BreathMark(), staff[3])
        >>> abjad.attach(abjad.BreathMark(), staff[7])
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            {
                c'8
                [
                d'8
                e'8
                f'8
                ]
                \breathe
                g'8
                [
                a'8
                b'8
                c''8
                ]
                \breathe
            }

    """

    ### CLASS VARIABLES ###

    __slots__ = (
        '_tweaks',
        )

    _format_slot = 'after'

    _time_orientation: enums.HorizontalAlignment = enums.Right

    ### INITIALIZER ###

    def __init__(
        self,
        *,
        tweaks: typing.Union[
            typing.List[typing.Tuple], LilyPondTweakManager] = None,
        ) -> None:
        self._tweaks = None
        LilyPondTweakManager.set_tweaks(self, tweaks)

    ### SPECIAL METHODS ###

    def __str__(self) -> str:
        r"""
        Gets string representation of breath mark.

        ..  container:: example

            >>> str(abjad.BreathMark())
            '\\breathe'

        """
        return r'\breathe'

    ### PRIVATE METHODS ###

    def _get_lilypond_format(self):
        return str(self)

    def _get_lilypond_format_bundle(self, component=None):
        bundle = LilyPondFormatBundle()
        if self.tweaks:
            tweaks = self.tweaks._list_format_contributions(directed=False)
            bundle.after.articulations.extend(tweaks)
        bundle.after.commands.append(self._get_lilypond_format())
        return bundle

    ### PUBLIC PROPERTIES ###

    @property
    def tweaks(self) -> typing.Optional[LilyPondTweakManager]:
        r"""
        Gets tweaks

        ..  container:: example

            >>> note = abjad.Note("c'4")
            >>> breath = abjad.BreathMark()
            >>> abjad.tweak(breath).color = 'blue'
            >>> abjad.attach(breath, note)
            >>> abjad.show(note) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(note)
                c'4
                \tweak color #blue
                \breathe

        ..  container:: example

            >>> note = abjad.Note("c'4")
            >>> breath = abjad.BreathMark(tweaks=[('color', 'blue')])
            >>> abjad.attach(breath, note)
            >>> abjad.show(note) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(note)
                c'4
                \tweak color #blue
                \breathe

        """
        return self._tweaks



from abjad.utilities.Enumerator import Enumerator
from .Set import Set


class IntervalSet(Set):
    """
    Interval set.
    """

    ### CLASS VARIABLES ###

    __slots__ = ()

    ### INITIALIZER ###

    def __init__(self, items=None, item_class=None):
        import abjad
        prototype = (
            abjad.PitchClassSegment,
            abjad.PitchClassSet,
            abjad.PitchSegment,
            abjad.PitchSet,
            )
        if isinstance(items, prototype):
            items = list(items)
            enumerator = Enumerator(items)
            pairs = enumerator.yield_pairs()
            items = [second - first for first, second in pairs]
        Set.__init__(
            self,
            items=items,
            item_class=item_class,
            )

    ### PRIVATE PROPERTIES ###

    @property
    def _named_item_class(self):
        import abjad
        return abjad.NamedInterval

    @property
    def _numbered_item_class(self):
        import abjad
        return abjad.NumberedInterval

    @property
    def _parent_item_class(self):
        import abjad
        return abjad.Interval

    ### PUBLIC METHODS ###

    @classmethod
    def from_selection(class_, selection, item_class=None):
        """
        Initializes interval set from component selection.

        ..  container:: example

            >>> staff_1 = abjad.Staff("c'4 <d' fs' a'>4 b2")
            >>> staff_2 = abjad.Staff("c4. r8 g2")
            >>> selection = abjad.select((staff_1, staff_2))
            >>> intervals = abjad.IntervalSet.from_selection(selection)
            >>> for interval in sorted(intervals):
            ...     interval
            ...
            NamedInterval('-M6')
            NamedInterval('-P5')
            NamedInterval('-A4')
            NamedInterval('-M3')
            NamedInterval('-m3')
            NamedInterval('-M2')
            NamedInterval('+m2')
            NamedInterval('+m3')
            NamedInterval('+M3')
            NamedInterval('+P4')
            NamedInterval('+P5')
            NamedInterval('+m7')
            NamedInterval('+M7')
            NamedInterval('+P8')
            NamedInterval('+M9')
            NamedInterval('+A11')
            NamedInterval('+M13')

        Returns interval set.
        """
        import abjad
        pitch_segment = abjad.PitchSegment.from_selection(selection)
        enumerator = Enumerator(pitch_segment)
        pairs = enumerator.yield_pairs()
        intervals = [second - first for first, second in pairs]
        return class_(
            items=intervals,
            item_class=item_class,
            )
